@@ -22,9 +22,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21283 $
+ * $Revision: 21988 $
  * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Date: 2009-02-04 06:28:03 +0000 (Wed, 04 Feb 2009) $
  */
 
 #ifndef OPAL_CODEC_G711CODEC_H
@@ -42,11 +42,9 @@
 
 ///////////////////////////////////////////////////////////////////////////////
 
-class Opal_G711_uLaw_PCM : public OpalStreamedTranscoder {
+class Opal_G711_PCM : public OpalStreamedTranscoder {
   public:
-    Opal_G711_uLaw_PCM();
-    virtual int ConvertOne(int sample) const;
-    static int ConvertSample(int sample);
+    Opal_G711_PCM(const OpalMediaFormat & inputMediaFormat);
 
 #if OPAL_G711PLC 
    virtual PBoolean Convert(
@@ -61,6 +59,16 @@ class Opal_G711_uLaw_PCM : public OpalStreamedTranscoder {
 };
 
 
+///////////////////////////////////////////////////////////////////////////////
+
+class Opal_G711_uLaw_PCM : public Opal_G711_PCM {
+  public:
+    Opal_G711_uLaw_PCM();
+    virtual int ConvertOne(int sample) const;
+    static int ConvertSample(int sample);
+};
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 class Opal_PCM_G711_uLaw : public OpalStreamedTranscoder {
@@ -73,22 +81,11 @@ class Opal_PCM_G711_uLaw : public OpalStreamedTranscoder {
 
 ///////////////////////////////////////////////////////////////////////////////
 
-class Opal_G711_ALaw_PCM : public OpalStreamedTranscoder {
+class Opal_G711_ALaw_PCM : public Opal_G711_PCM {
   public:
     Opal_G711_ALaw_PCM();
     virtual int ConvertOne(int sample) const;
     static int ConvertSample(int sample);
-
-#if OPAL_G711PLC 
-   virtual PBoolean Convert(
-      const RTP_DataFrame & input,  ///<  Input data
-      RTP_DataFrame & output        ///<  Output data
-    );
-
-  protected:
-    OpalG711_PLC plc;
-    PINDEX       lastPayloadSize;
-#endif
 };
 
 
