@@ -27,9 +27,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 21283 $
+ * $Revision: 22300 $
  * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Date: 2009-03-26 03:40:31 +0000 (Thu, 26 Mar 2009) $
  */
 
 #ifndef OPAL_OPAL_CONNECTION_H
@@ -48,6 +48,7 @@
 #include <ptclib/dtmf.h>
 #include <ptlib/safecoll.h>
 #include <rtp/rtp.h>
+#include <im/t140.h>
 
 class OpalEndPoint;
 class OpalCall;
@@ -56,6 +57,26 @@ class OpalEchoCanceler;
 class OpalRFC2833Proto;
 class OpalRFC2833Info;
 
+
+#define OPAL_OPT_AUTO_START           "AutoStart"             ///< String option for auto-started media types
+#define OPAL_OPT_CALL_IDENTIFIER      "Call-Identifier"       ///< String option to override generated call identifier
+#define OPAL_OPT_CALLING_PARTY_URL    "Calling-Party-URL"     ///< String option to set outgoing local URL
+#define OPAL_OPT_CALLING_PARTY_NUMBER "Calling-Party-Number"  ///< String option to set outgoing local number
+#define OPAL_OPT_CALLING_PARTY_NAME   "Calling-Party-Name"    ///< String option to set outgoing local name
+#define OPAL_OPT_CALLING_PARTY_DOMAIN "Calling-Party-Domain"  ///< String option to set outgoing local host/address/domain
+#define OPAL_OPT_CALLING_DISPLAY_NAME "Calling-Display-Name"  ///< String option to set outgoing display name
+#define OPAL_OPT_ORIGINATOR_ADDRESS   "Originator-Address"    ///< String option for originator address used by IVR
+#define OPAL_OPT_INTERFACE            "Interface"             ///< String option to set the interface used for outgoing call
+#define OPAL_OPT_ENABLE_INBAND_DTMF   "EnableInbandDTMF"      ///< String option to enable in band DTMF detection
+#define OPAL_OPT_DTMF_MULT            "dtmfmult"
+#define OPAL_OPT_DTMF_DIV             "dtmfdiv"
+#define OPAL_OPT_DISABLE_JITTER       "Disable-Jitter"        ///< String option to disable jitter buffer if "true"
+#define OPAL_OPT_MAX_JITTER           "Max-Jitter"            ///< String option to set maximum jitter in milliseconds
+#define OPAL_OPT_MIN_JITTER           "Min-Jitter"            ///< String option to set minimum jitter in milliseconds
+#define OPAL_OPT_RECORD_AUDIO         "Record-Audio"          ///< String option to start recording to a file for call
+#define OPAL_OPT_ALERTING_TYPE        "Alerting-Type"         ///< String option to set teh alerting type string for call
+
+
 /*! \page pageOpalConnections Connection handling in the OPAL library
 
 \section secOverview Overview 
@@ -450,14 +471,15 @@ class OpalConnection : public PSafeObject
        instances. These phases are fully described in the documentation page
        \ref pageOpalConnections.  */
     enum Phases {
-	UninitialisedPhase,   /*!< Indicates the OpalConnection instance has just been constructed*/
-	SetUpPhase,           /*!< In the process of sending/receiving the initial INVITE packet */
-	AlertingPhase,        /*!< There is a phone ringing, somewhere                           */
-	ConnectedPhase,       /*!< There is agreement on having a call, media type etc            */
-	EstablishedPhase,     /*!< Media is flowing, control streams  are all operational         */
-	ReleasingPhase,       /*!< Hangup packet has been sent/received, media and control not yet stopped  */
-	ReleasedPhase,        /*!< Media and control streams have been terminated                           */
-	NumPhases             /*!< Number of available phases. Can be used to indicate an unknown phase     */
+      UninitialisedPhase,   //!< Indicates the OpalConnection instance has just been constructed
+      SetUpPhase,           //!< In the process of sending/receiving the initial INVITE packet
+      ProceedingPhase,      //!< The remote is now responsible for completing the call
+      AlertingPhase,        //!< The remote says there is a phone ringing, somewhere
+      ConnectedPhase,       //!< There is agreement on having a call, usually means billing will apply
+      EstablishedPhase,     //!< Media is flowing, control streams  are all operational
+      ReleasingPhase,       //!< Hangup packet has been sent/received, media and control not yet stopped
+      ReleasedPhase,        //!< Media and control streams have been terminated
+      NumPhases             //!< Number of available phases. Can be used to indicate an unknown phase
     };
 
     /**Get the phase of the connection.
@@ -590,18 +612,29 @@ class OpalConnection : public PSafeObject
     virtual PBoolean OnSetUpConnection();
 
     
+    /**Call back for remote party is now responsible for completing the call.
+       This function is called when the remote system has been contacted and it
+       has accepted responsibility for completing, or failing, the call. This
+       is distinct from OnAlerting() in that it is not known at this time if
+       anything is ringing. This indication may be used to distinguish between
+       "transport" level error, in which case another host may be tried, and
+       that finalising the call has moved "upstream" and the local system has
+       no more to do but await a result.
+
+       If an application overrides this function, it should generally call the
+       ancestor version for correct operation.
+
+       The default behaviour calls the OpalEndPoint function of the same name.
+     */
+    virtual void OnProceeding();
+
     /**Call back for remote party being alerted.
        This function is called after the connection is informed that the
        remote endpoint is "ringing". Generally some time after the
        SetUpConnection() function was called, this is function is called.
 
-       If PFalse is returned the connection is aborted.
-
        If an application overrides this function, it should generally call the
-       ancestor version for correct operation. An application would typically
-       only intercept this function if it wishes to do some form of logging.
-       For this you can obtain the name of the caller by using the function
-       OpalConnection::GetRemotePartyName().
+       ancestor version for correct operation.
 
        The default behaviour calls the OpalEndPoint function of the same name.
      */
@@ -861,7 +894,8 @@ class OpalConnection : public PSafeObject
        Locates a stream given an identifier string.
       */
     OpalMediaStreamPtr GetMediaStream(
-      const PString & streamID  ///<  Stream ID to search for.
+      const PString & streamID,  ///<  Stream ID to search for.
+      bool source                ///<  Indicates the direction of stream.
     ) const;
 
     /**Get a media stream.
@@ -1289,6 +1323,38 @@ class OpalConnection : public PSafeObject
      */
     void CopyPartyNames(const OpalConnection & other);
 
+
+    /**Get alerting type information of an incoming call.
+       The type of "distinctive ringing" for the call. The string is protocol
+       dependent, so the caller would need to be aware of the type of call
+       being made. Some protocols may ignore the field completely.
+
+       For SIP this corresponds to the string contained in the "Alert-Info"
+       header field of the INVITE. This is typically a URI for the ring file.
+
+       For H.323 this must be a string representation of an integer from 0 to 7
+       which will be contained in the Q.931 SIGNAL (0x34) Information Element.
+
+       Default behaviour returns an empty string.
+      */
+    virtual PString GetAlertingType() const;
+
+    /**Set alerting type information for outgoing call.
+       The type of "distinctive ringing" for the call. The string is protocol
+       dependent, so the caller would need to be aware of the type of call
+       being made. Some protocols may ignore the field completely.
+
+       For SIP this corresponds to the string contained in the "Alert-Info"
+       header field of the INVITE. This is typically a URI for the ring file.
+
+       For H.323 this must be a string representation of an integer from 0 to 7
+       which will be contained in the Q.931 SIGNAL (0x34) Information Element.
+
+       Default behaviour returns false.
+      */
+    virtual bool SetAlertingType(const PString & info);
+
+
     /**Get the default maximum audio jitter delay parameter.
        Defaults to 50ms
      */
@@ -1315,9 +1381,9 @@ class OpalConnection : public PSafeObject
     OpalEchoCanceler * GetEchoCanceler() const { return echoCanceler; }
 
     /**Get the protocol-specific unique identifier for this connection.
+       Default behaviour just returns the connection token.
      */
-    virtual const OpalGloballyUniqueID & GetIdentifier() const
-    { return callIdentifier; }
+    virtual PString GetIdentifier() const;
 
     /**Get the maximum RTP payload size. This function allows a user to
        override the value returned on a connection by connection basis, for
@@ -1343,25 +1409,17 @@ class OpalConnection : public PSafeObject
 #endif
   //@}
 
-    StringOptions * GetStringOptions() const
-    { return stringOptions; }
+    /// Get the string options associated with this connection.
+    const StringOptions & GetStringOptions() const { return m_connStringOptions; }
 
-    void SetStringOptions(StringOptions * options);
+    /// Set the string options associated with this connection.
+    void SetStringOptions(
+      const StringOptions & options,
+      bool overwrite
+    );
 
-    /**Open the media channels associated with an incoming call
- 
-       This function is provided to allow an OpalConnection descendant to delay the opening of the
-       media channels associated with an incoming call. By default, this function is called as soon
-       as possible after an incoming connection request has been received (i.e. an INVITE or SETUP) and
-       and do all of the work associated with handling SDP or fastStart eyc
-
-       By overriding this function, the media channel open is deferred, which gives the other connections
-       in the call a chance to do something before the media channels are started. This allows for features
-       beyond those provided by the OnAnswer interface
-      */
-    virtual PBoolean OnOpenIncomingMediaChannels();
-
-    virtual void ApplyStringOptions();
+    virtual void ApplyStringOptions(OpalConnection::StringOptions & stringOptions);
+    virtual void OnApplyStringOptions();
 
     virtual void PreviewPeerMediaFormats(const OpalMediaFormatList & fmts);
 
@@ -1382,6 +1440,32 @@ class OpalConnection : public PSafeObject
       */
     PDECLARE_NOTIFIER(OpalMediaCommand, OpalConnection, OnMediaCommand);
 
+    /** Indicate whether a particular media type can auto-start.
+        This is typically used for things like video or fax to contol if on
+        initial connection, that media type is opened straight away. Streams
+        of that type may be opened later, during the call, by using the
+        OpalCall::OpenSourceMediaStreams() function.
+      */
+    virtual OpalMediaType::AutoStartMode GetAutoStart(
+      const OpalMediaType & mediaType  ///< media type to check
+    ) const;
+
+#if OPAL_HAS_IM
+    struct IMInfo : public PObject {
+      unsigned sessionId;
+      OpalMediaFormat mediaFormat;
+      T140String body;
+    };
+
+    virtual bool SendIM(const OpalMediaFormat & format, const T140String & body);
+    virtual bool SendIM(const OpalMediaFormat & format, const RTP_DataFrame & body);
+    virtual void OnReceiveIM(const IMInfo & im);
+
+    void AddIMListener(
+      const PNotifier & listener
+    );
+#endif
+
   protected:
     void OnConnectedInternal();
 
@@ -1401,7 +1485,6 @@ class OpalConnection : public PSafeObject
 
   protected:
     PString              callToken;
-    OpalGloballyUniqueID callIdentifier;
     PBoolean             originating;
     PTime                setupTime;
     PTime                alertingTime;
@@ -1452,12 +1535,38 @@ class OpalConnection : public PSafeObject
     friend ostream & operator<<(ostream & o, Phases p);
 #endif
 
-    StringOptions * stringOptions;
+    StringOptions m_connStringOptions;
     PString recordAudioFilename;
+    PNotifier recordNotifier;
 
 #if OPAL_STATISTICS
     unsigned m_VideoUpdateRequestsSent;
 #endif
+
+    struct AutoStartInfo {
+      unsigned preferredSessionId;  // preferred session ID (only used for originating)
+      OpalMediaType::AutoStartMode autoStart;// Mode for this session when the call is started
+    };
+
+    class AutoStartMap : public std::map<OpalMediaType, AutoStartInfo>
+    {
+      public:
+        AutoStartMap();
+        void Initialise(const OpalConnection::StringOptions & stringOptions);
+        OpalMediaType::AutoStartMode GetAutoStart(const OpalMediaType & mediaType) const;
+        void SetAutoStart(const OpalMediaType & mediaType, OpalMediaType::AutoStartMode autoStart);
+
+      protected:
+        bool m_initialised;
+        PMutex m_mutex;
+
+    };
+    AutoStartMap m_autoStartInfo;
+
+#if OPAL_HAS_IM
+    mutable PList<PNotifier> m_imListeners;
+#endif
+
 };
 
 #endif // OPAL_OPAL_CONNECTION_H
