@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21848 $
- * $Author: csoutheren $
- * $Date: 2008-12-22 00:47:09 +0000 (Mon, 22 Dec 2008) $
+ * $Revision: 22680 $
+ * $Author: rjongbloed $
+ * $Date: 2009-05-21 00:48:57 +0000 (Thu, 21 May 2009) $
  */
 
 #ifndef OPAL_OPAL_MEDIAFMT_H
@@ -128,10 +128,19 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
       const char * protocol = NULL                ///<  protocol to be valid for (if NULL, then all)
     ) const;
 
-    /**Get a format position in the list matching the wildcard.
-       The wildcard string is a simple substring match using the '*'
-       character. For example: "G.711*" would match "G.711-uLaw-64k" and
-       "G.711-ALaw-64k".
+    /**Get a position in the list of the first entry matching the wildcard.
+       The wildcard string is a string match using several special
+       characters.
+
+       The '*' character indicates substrings, for example: "G.711*" would
+       match "G.711-uLaw-64k" and "G.711-ALaw-64k".
+
+       The '@' character indicates a type of media format, so say "@video
+       would find the first video codec.
+
+       The '!' character indicates a negative test. That is the first entry
+       that does NOT match the string is returned. The string after the '!'
+       may contain '*' and '@' characters.
 
        Returns P_MAX_INDEX if not in list.
       */
@@ -140,18 +149,6 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
       const_iterator start = const_iterator()
     ) const;
 
-    /**Get a format position in the list not matching the wildcard.
-       The wildcard string is a simple substring match using the '*'
-       character. For example: "G.711*" would match "G.711-uLaw-64k" and
-       "G.711-ALaw-64k".
-
-       Returns P_MAX_INDEX list contains only matches.
-      */
-    const_iterator FindNotFormat(
-      const PString & wildcard,    ///<  Wildcard string name.
-      const_iterator start = const_iterator()
-    ) const;
-
     /**Determine if a format matching the payload type is in the list.
       */
     PBoolean HasFormat(
@@ -168,14 +165,41 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
     ) const { return FindFormat(wildcard) != end(); }
 
     /**Remove all the formats specified.
+       Each string in the array is checked using the wildcard matching algorithm in
+       FindFormat().
+
+       The '*' character indicates substrings, for example: "G.711*" would
+       match "G.711-uLaw-64k" and "G.711-ALaw-64k".
+
+       The '@' character indicates a type of media format, so say "@video
+       would find the first video codec.
+
+       The '!' character indicates a negative test. That is the first entry
+       that does NOT match the string is returned. The string after the '!'
+       may contain '*' and '@' characters.
+
+       I should be noted that when the ! operator is used, codecs are removed
+       at that time and later ! operators may not have an affect. For example
+       {"!G.729","!G.728"} will NOT give you two codecs but on G.729.
       */
     void Remove(
       const PStringArray & mask
     );
 
     /**Reorder the formats in the list.
-       The order variable is an array of wildcards and the list is reordered
+       The order variable is an array of names and the list is reordered
        according to the order in that array.
+
+       Each string in the array is checked using the wildcard matching algorithm
+       similar to FindFormat().
+
+       The '*' character indicates substrings, for example: "G.711*" would
+       match "G.711-uLaw-64k" and "G.711-ALaw-64k".
+
+       The '@' character indicates a type of media format, so say "@video
+       would sort by video codec.
+
+        The '!' character is not supported.
       */
     void Reorder(
       const PStringArray & order
@@ -280,7 +304,7 @@ class OpalMediaOption : public PObject
     };
 
     const H245GenericInfo & GetH245Generic() const { return m_H245Generic; }
-    void SetH245Generic(const H245GenericInfo & generic) { m_H245Generic = generic; }
+    void SetH245Generic(const H245GenericInfo & genericInfo) { m_H245Generic = genericInfo; }
 #endif // OPAL_H323
 
   protected:
@@ -779,8 +803,8 @@ class OpalMediaFormat : public PContainer
     static const PString & TargetBitRateOption();
 
     /**Get the maximum frame size in bytes. If this returns zero then the
-       media format has no intrinsic maximum frame size, eg G.711 would 
-       return zero but G.723.1 whoud return 24.
+       media format has no intrinsic maximum frame size, eg a video format
+       would return zero but G.723.1 would return 24.
       */
     PINDEX GetFrameSize() const { PWaitAndSignal m(_mutex); return m_info == NULL ? 0 : m_info->GetOptionInteger(MaxFrameSizeOption(), 0); }
     static const PString & MaxFrameSizeOption();
@@ -805,6 +829,14 @@ class OpalMediaFormat : public PContainer
     unsigned GetClockRate() const { PWaitAndSignal m(_mutex); return m_info == NULL ? 0 : m_info->GetOptionInteger(ClockRateOption(), 1000); }
     static const PString & ClockRateOption();
 
+    /**Get the name of the OpalMediaOption indicating the protocol the format is being used on.
+      */
+    static const PString & ProtocolOption();
+
+    /**Get the name of the OpalMediaOption indicating the maximum RTP payload size.
+      */
+    static const PString & MaxTxPacketSizeOption();
+
     /**Get all of the option values of the format as a dictionary.
        Each entry is a name value pair.
       */
@@ -1019,15 +1051,15 @@ class OpalMediaFormat : public PContainer
     operator PString() const { PWaitAndSignal m(_mutex); return m_info == NULL ? "" : m_info->formatName; }
     operator const char *() const { PWaitAndSignal m(_mutex); return m_info == NULL ? "" : m_info->formatName; }
     bool operator==(const char * other) const { PWaitAndSignal m(_mutex); return m_info != NULL && m_info->formatName == other; }
-    bool operator!=(const char * other) const { PWaitAndSignal m(_mutex); return m_info == NULL || m_info->formatName == other; }
+    bool operator!=(const char * other) const { PWaitAndSignal m(_mutex); return m_info == NULL || m_info->formatName != other; }
     bool operator==(const PString & other) const { PWaitAndSignal m(_mutex); return m_info != NULL && m_info->formatName == other; }
-    bool operator!=(const PString & other) const { PWaitAndSignal m(_mutex); return m_info == NULL || m_info->formatName == other; }
+    bool operator!=(const PString & other) const { PWaitAndSignal m(_mutex); return m_info == NULL || m_info->formatName != other; }
     bool operator==(const OpalMediaFormat & other) const { PWaitAndSignal m(_mutex); return Compare(other) == EqualTo; }
     bool operator!=(const OpalMediaFormat & other) const { PWaitAndSignal m(_mutex); return Compare(other) != EqualTo; }
     friend bool operator==(const char * other, const OpalMediaFormat & fmt) { return fmt.m_info != NULL && fmt.m_info->formatName == other; }
-    friend bool operator!=(const char * other, const OpalMediaFormat & fmt) { return fmt.m_info == NULL || fmt.m_info->formatName == other; }
+    friend bool operator!=(const char * other, const OpalMediaFormat & fmt) { return fmt.m_info == NULL || fmt.m_info->formatName != other; }
     friend bool operator==(const PString & other, const OpalMediaFormat & fmt) { return fmt.m_info != NULL && fmt.m_info->formatName == other; }
-    friend bool operator!=(const PString & other, const OpalMediaFormat & fmt) { return fmt.m_info == NULL || fmt.m_info->formatName == other; }
+    friend bool operator!=(const PString & other, const OpalMediaFormat & fmt) { return fmt.m_info == NULL || fmt.m_info->formatName != other; }
 
 #if OPAL_H323
     static const PString & MediaPacketizationOption();
@@ -1132,8 +1164,8 @@ class OpalVideoFormat : public OpalMediaFormat
     static const PString & TemporalSpatialTradeOffOption();
     static const PString & TxKeyFramePeriodOption();
     static const PString & RateControlEnableOption();
-    static const PString & RateControlWindowSizeOption();
-    static const PString & RateControlMaxFramesSkipOption();
+    static const PString & RateControllerOption();
+    static const PString & RateControllerBitRateScalerOption();
 };
 #endif
 
@@ -1149,6 +1181,9 @@ class OpalVideoFormat : public OpalMediaFormat
 #define OPAL_L16_MONO_48KHZ "Linear-16-Mono-48kHz"
 #define OPAL_G711_ULAW_64K  "G.711-uLaw-64k"
 #define OPAL_G711_ALAW_64K  "G.711-ALaw-64k"
+#define OPAL_G722           "G.722"
+#define OPAL_G7221          "G.722.1"
+#define OPAL_G7222          "G.722.2"
 #define OPAL_G726_40K       "G.726-40K"
 #define OPAL_G726_32K       "G.726-32K"
 #define OPAL_G726_24K       "G.726-24K"
@@ -1185,6 +1220,9 @@ extern const OpalAudioFormat & GetOpalL16_MONO_48KHZ();
 extern const OpalAudioFormat & GetOpalL16_STEREO_48KHZ();
 extern const OpalAudioFormat & GetOpalG711_ULAW_64K();
 extern const OpalAudioFormat & GetOpalG711_ALAW_64K();
+extern const OpalAudioFormat & GetOpalG722();
+extern const OpalAudioFormat & GetOpalG7221();
+extern const OpalMediaFormat & GetOpalG7222();
 extern const OpalAudioFormat & GetOpalG726_40K();
 extern const OpalAudioFormat & GetOpalG726_32K();
 extern const OpalAudioFormat & GetOpalG726_24K();
@@ -1199,7 +1237,7 @@ extern const OpalAudioFormat & GetOpalG7231_5k3();
 extern const OpalAudioFormat & GetOpalG7231A_6k3();
 extern const OpalAudioFormat & GetOpalG7231A_5k3();
 extern const OpalAudioFormat & GetOpalGSM0610();
-extern const OpalAudioFormat & GetOpalGSMAMR();
+extern const OpalMediaFormat & GetOpalGSMAMR();
 extern const OpalMediaFormat & GetOpaliLBC();
 
 extern const OpalMediaFormat & GetOpalRFC2833();
@@ -1221,6 +1259,9 @@ extern const OpalMediaFormat & GetOpalT38();
 #define OpalL16_STEREO_48KHZ GetOpalL16_STEREO_48KHZ()
 #define OpalG711_ULAW_64K  GetOpalG711_ULAW_64K()
 #define OpalG711_ALAW_64K  GetOpalG711_ALAW_64K()
+#define OpalG722           GetOpalG722()
+#define OpalG7221          GetOpalG7221()
+#define OpalG7222          GetOpalG7222()
 #define OpalG726_40K       GetOpalG726_40K()
 #define OpalG726_32K       GetOpalG726_32K()
 #define OpalG726_24K       GetOpalG726_24K()
