@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21807 $
- * $Author: csoutheren $
- * $Date: 2008-12-16 03:28:12 +0000 (Tue, 16 Dec 2008) $
+ * $Revision: 22994 $
+ * $Author: rjongbloed $
+ * $Date: 2009-06-26 06:07:27 +0000 (Fri, 26 Jun 2009) $
  */
 
 #ifndef OPAL_SIP_SIPEP_H
@@ -44,7 +44,7 @@
 
 #if OPAL_SIP
 
-#include <ptclib/sockagg.h>
+#include <ptclib/threadpool.h>
 #include <opal/rtpep.h>
 #include <sip/sipcon.h>
 #include <sip/sippdu.h>
@@ -57,11 +57,6 @@
 class SIPRegisterHandler;
 
 
-//
-//  provide flag so applications know if presence is available
-//
-#define OPAL_HAS_SIP_PRESENCE   1
-
 /////////////////////////////////////////////////////////////////////////
 
 /** Class to hold authentication information 
@@ -159,6 +154,28 @@ class SIPEndPoint : public OpalRTPEndPoint
       OpalConnection::StringOptions * stringOptions  ///<  complex string options
     );
 
+    /**A call back function whenever a connection is broken.
+       This function can do any internal cleaning up and waiting on background
+       threads that may be using the connection object.
+
+       Note that there is not a one to one relationship with the
+       OnEstablishedConnection() function. This function may be called without
+       that function being called. For example if MakeConnection() was used
+       but the call never completed.
+
+       Classes that override this function should make sure they call the
+       ancestor version for correct operation.
+
+       An application will not typically call this function as it is used by
+       the OpalManager during a release of the connection.
+
+       The default behaviour removes the connection from the internal database
+       and calls the OpalManager function of the same name.
+      */
+    virtual void OnReleased(
+      OpalConnection & connection   ///<  Connection that was established
+    );
+
     /** Execute garbage collection for endpoint.
         Returns PTrue if all garbage has been collected.
         Default behaviour deletes the objects in the connectionsActive list.
@@ -325,11 +342,15 @@ class SIPEndPoint : public OpalRTPEndPoint
     /**Find a connection that uses the specified token.
        This searches the endpoint for the connection that contains the token
        as provided by functions such as MakeConnection().
+
+       Note the token may be a "replaces" style value of the form:
+         "callid;to-tag=tag;from-tag=tag"
       */
     PSafePtr<SIPConnection> GetSIPConnectionWithLock(
       const PString & token,     ///<  Token to identify connection
-      PSafetyMode mode = PSafeReadWrite
-    ) { return PSafePtrCast<OpalConnection, SIPConnection>(GetConnectionWithLock(token, mode)); }
+      PSafetyMode mode = PSafeReadWrite,
+      SIP_PDU::StatusCodes * errorCode = NULL
+    );
 
     virtual PBoolean IsAcceptedAddress(const SIPURL & toAddr);
 
@@ -376,6 +397,7 @@ class SIPEndPoint : public OpalRTPEndPoint
     bool Unregister(const PString & aor);
 
     /**Unregister all current registrations.
+       Returns true if at least one registra is unregistered.
       */
     bool UnregisterAll();
 
@@ -407,6 +429,7 @@ class SIPEndPoint : public OpalRTPEndPoint
       bool                 m_reRegistering;     ///< Was a registration refresh
       SIP_PDU::StatusCodes m_reason;            ///< Reason for status change
       OpalProductInfo      m_productInfo;       ///< Server product info from registrar if available.
+      void               * m_userData;          ///< User data corresponding to this registration
     };
 
     /**Callback called when a registration to a SIP registrar status.
@@ -454,7 +477,8 @@ class SIPEndPoint : public OpalRTPEndPoint
       PString & aor                        ///< Resultant address-of-record for unsubscribe
     );
 
-
+    /**Unsubscribe a current subscriptions.
+      */
     bool Unsubscribe(
       SIPSubscribe::PredefinedPackages eventPackage,  ///< Event package being unsubscribed
       const PString & aor                             ///< Address-of-record for subscription
@@ -465,6 +489,7 @@ class SIPEndPoint : public OpalRTPEndPoint
     );
 
     /**Unsubscribe all current subscriptions.
+       Returns true if at least one subscription is unsubscribed.
       */
     bool UnsubcribeAll(
       SIPSubscribe::PredefinedPackages eventPackage  ///< Event package being unsubscribed
@@ -519,7 +544,7 @@ class SIPEndPoint : public OpalRTPEndPoint
     bool Notify(
       const SIPURL & targetAddress, ///< Address that was subscribed
       const PString & eventPackage, ///< Event package for notification
-      const PString & body          ///< Body of notification
+      const PObject & body          ///< Body of notification
     );
 
 
@@ -693,7 +718,7 @@ class SIPEndPoint : public OpalRTPEndPoint
      * If no active registration is used, return the result of GetLocalURL
      * on the given transport.
      */
-    SIPURL GetContactURL(const OpalTransport &transport, const PString & userName, const PString & host);
+    SIPURL GetContactURL(const OpalTransport &transport, const SIPURL & localURI);
 
 
     /**Return the local URL for the given transport and user name.
@@ -799,46 +824,53 @@ class SIPEndPoint : public OpalRTPEndPoint
 
     bool              m_shuttingDown;
     SIPHandlersList   activeSIPHandlers;
+    PStringToString   m_receivedConnectionTokens;
 
     PSafeDictionary<PString, SIPTransaction> transactions;
 
     PTimer                  natBindingTimer;
     NATBindingRefreshMethod natMethod;
     PAtomicInteger          lastSentCSeq;
-    unsigned                m_dialogNotifyVersion;
     int                     m_defaultAppearanceCode;
 
     struct SIP_PDU_Work
     {
-      SIP_PDU_Work()
-      { ep = NULL; pdu = NULL; }
+      public:
+        SIP_PDU_Work(SIPEndPoint & ep, const PString & token, SIP_PDU * pdu);
+        ~SIP_PDU_Work();
 
-      SIPEndPoint * ep;
-      SIP_PDU * pdu;
-      PString callID;
+        void OnReceivedPDU();
+
+        SIPEndPoint & m_endpoint;
+        PString       m_token;
+        SIP_PDU     * m_pdu;
     };
 
+    class PDUThreadPool : public PThreadPool<SIP_PDU_Work>
+    {
+      public:
+        virtual WorkerThreadBase * CreateWorkerThread();
+    } m_connectionThreadPool, m_handlerThreadPool;
+
     typedef std::queue<SIP_PDU_Work *> SIP_PDUWorkQueue;
 
-    class SIP_PDU_Thread : public PThreadPoolWorkerBase
+    class SIP_PDU_Thread : public PDUThreadPool::WorkerThread
     {
       public:
-        SIP_PDU_Thread(PThreadPoolBase & _pool);
+        SIP_PDU_Thread(PDUThreadPool & pool_);
+
+        void AddWork(SIP_PDU_Work * work);
+        void RemoveWork(SIP_PDU_Work * work);
         unsigned GetWorkSize() const;
-        void OnAddWork(SIP_PDU_Work * work);
-        void OnRemoveWork(SIP_PDU_Work *);
-        void Shutdown();
+
         void Main();
+        void Shutdown();
 
       protected:
-        PMutex mutex;
-        PSyncPoint sync;
-        SIP_PDUWorkQueue pduQueue;
+        PSyncPoint m_sync;
+        SIP_PDUWorkQueue m_pduQueue;
     };
 
-    typedef PThreadPool<SIP_PDU_Work, SIP_PDU_Thread> SIPMainThreadPool;
-    SIPMainThreadPool threadPool;
-
     enum {
       HighPriority = 80,
       LowPriority  = 30,
@@ -852,7 +884,7 @@ class SIPEndPoint : public OpalRTPEndPoint
         virtual void OnAddInterface(const PIPSocket::InterfaceEntry & entry);
         virtual void OnRemoveInterface(const PIPSocket::InterfaceEntry & entry);
 
-    protected:
+      protected:
         SIPEndPoint & m_endpoint;
     };
     InterfaceMonitor m_highPriorityMonitor;
