@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22097 $
+ * $Revision: 22928 $
  * $Author: rjongbloed $
- * $Date: 2009-02-21 04:55:10 +0000 (Sat, 21 Feb 2009) $
+ * $Date: 2009-06-19 05:26:12 +0000 (Fri, 19 Jun 2009) $
  */
 
 #ifndef OPAL_SIP_SIPPDU_H
@@ -248,10 +248,10 @@ class SIPMIMEInfo : public PMIMEInfo
 
     void SetCompactForm(bool form) { compactForm = form; }
 
-    PString GetContentType() const;
+    PCaselessString GetContentType() const;
     void SetContentType(const PString & v);
 
-    PString GetContentEncoding() const;
+    PCaselessString GetContentEncoding() const;
     void SetContentEncoding(const PString & v);
 
     PString GetFrom() const;
@@ -742,6 +742,7 @@ class SIP_PDU : public PSafeObject
           SIPMIMEInfo & GetMIME()            { return mime; }
     void SetURI(const SIPURL & newuri)       { uri = newuri; }
     SDPSessionDescription * GetSDP();
+    void SetSDP(SDPSessionDescription * sdp);
 
   protected:
     Methods     method;                 // Request type, ==NumMethods for Response
@@ -945,6 +946,7 @@ class SIPInvite : public SIPTransaction
     virtual PBoolean OnReceivedResponse(SIP_PDU & response);
 
     const OpalRTPSessionManager & GetSessionManager() const { return rtpSessions; }
+          OpalRTPSessionManager & GetSessionManager()       { return rtpSessions; }
 
   protected:
     OpalRTPSessionManager rtpSessions;
@@ -970,6 +972,7 @@ class SIPRegister : public SIPTransaction
       unsigned      m_restoreTime;
       PTimeInterval m_minRetryTime;
       PTimeInterval m_maxRetryTime;
+      void          * m_userData;
     };
 
     SIPRegister(
@@ -1025,6 +1028,8 @@ class SIPSubscribe : public SIPTransaction
       unsigned      m_restoreTime;
       PTimeInterval m_minRetryTime;
       PTimeInterval m_maxRetryTime;
+      void          * m_userData;
+      PString       m_from;
     };
 
     SIPSubscribe(
@@ -1047,7 +1052,7 @@ class SIPEventPackageHandler
 {
 public:
   virtual ~SIPEventPackageHandler() { }
-  virtual PString GetContentType() const = 0;
+  virtual PCaselessString GetContentType() const = 0;
   virtual bool OnReceivedNOTIFY(SIPHandler & handler, SIP_PDU & request) = 0;
   virtual PString OnSendNOTIFY(SIPHandler & /*handler*/, const PObject * /*body*/) { return PString::Empty(); }
 };
