@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 22351 $
+ * $Author: csoutheren $
+ * $Date: 2009-03-31 07:10:48 +0000 (Tue, 31 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -96,7 +96,9 @@ OpalVideoTranscoder::OpalVideoTranscoder(const OpalMediaFormat & inputMediaForma
   : OpalTranscoder(inputMediaFormat, outputMediaFormat)
   , inDataSize(10*1024)
   , outDataSize(10*1024)
+  , videoEncoderMaxOutputSize(10*1024)
   , forceIFrame(true)
+  , lastFrameWasIFrame(false)
 #if OPAL_STATISTICS
   , m_totalFrames(0)
   , m_keyFrames(0)
@@ -125,6 +127,8 @@ bool OpalVideoTranscoder::UpdateMediaFormats(const OpalMediaFormat & input, cons
   SetFrameBytes(inputMediaFormat,  OpalVideoFormat::MaxRxFrameWidthOption(), OpalVideoFormat::MaxRxFrameHeightOption(), inDataSize);
   SetFrameBytes(outputMediaFormat, OpalVideoFormat::FrameWidthOption(),      OpalVideoFormat::FrameHeightOption(),      outDataSize);
 
+  videoEncoderMaxOutputSize = outputMediaFormat.GetOptionInteger(OpalVideoFormat::MaxTxPacketSizeOption(), outDataSize);
+
   return PTrue;
 }
 
