@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22032 $
+ * $Revision: 22197 $
  * $Author: rjongbloed $
- * $Date: 2009-02-11 01:28:11 +0000 (Wed, 11 Feb 2009) $
+ * $Date: 2009-03-12 01:11:14 +0000 (Thu, 12 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -1099,7 +1099,7 @@ void H323DataChannel::OnSendOpenAck(const H245_OpenLogicalChannel & open,
   H323TransportAddress address;
   param->IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaChannel);
   if (listener != NULL)
-    address = listener->GetLocalAddress(connection.GetControlChannel().GetLocalAddress());
+    address = listener->GetLocalAddress(connection.GetControlChannel().GetRemoteAddress());
   else
     address = transport->GetLocalAddress();
 
