@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20352 $
+ * $Revision: 21004 $
  * $Author: rjongbloed $
- * $Date: 2008-06-01 09:30:08 +0200 (dim 01 jun 2008) $
+ * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -50,7 +50,7 @@
 
 #include <ptclib/random.h>
 
-#ifdef OPAL_H460
+#if OPAL_H460
 #include <h460/h460.h>
 #endif
 
@@ -427,7 +427,7 @@ void H225_RAS::OnSendGatekeeperRequest(H323RasPDU &, H225_GatekeeperRequest & gr
     grq.m_gatekeeperIdentifier = gatekeeperIdentifier;
   }
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   SendFeatureSet<H225_GatekeeperRequest>(this, H460_MessageType::e_gatekeeperRequest, grq);
 #endif
 
@@ -447,7 +447,7 @@ void H225_RAS::OnSendGatekeeperConfirm(H323RasPDU &, H225_GatekeeperConfirm & gc
     gcf.m_gatekeeperIdentifier = gatekeeperIdentifier;
   }
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   SendFeatureSet<H225_GatekeeperConfirm>(this, H460_MessageType::e_gatekeeperConfirm, gcf);
 #endif
 
@@ -467,7 +467,7 @@ void H225_RAS::OnSendGatekeeperReject(H323RasPDU &, H225_GatekeeperReject & grj)
     grj.m_gatekeeperIdentifier = gatekeeperIdentifier;
   }
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   SendFeatureSet<H225_GatekeeperReject>(this, H460_MessageType::e_gatekeeperReject, grj);
 #endif
 
@@ -489,7 +489,7 @@ static void ReceiveFeatureSet(const H225_RAS * ras, unsigned code, const PDUType
 
 PBoolean H225_RAS::OnReceiveGatekeeperRequest(const H323RasPDU &, const H225_GatekeeperRequest & grq)
 {
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_GatekeeperRequest>(this, H460_MessageType::e_gatekeeperRequest, grq);
 #endif
 
@@ -521,7 +521,7 @@ PBoolean H225_RAS::OnReceiveGatekeeperConfirm(const H323RasPDU &, const H225_Gat
     }
   }
   
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_GatekeeperConfirm>(this, H460_MessageType::e_gatekeeperConfirm, gcf);
 #endif
 
@@ -540,7 +540,7 @@ PBoolean H225_RAS::OnReceiveGatekeeperReject(const H323RasPDU &, const H225_Gate
   if (!CheckForResponse(H225_RasMessage::e_gatekeeperRequest, grj.m_requestSeqNum, &grj.m_rejectReason))
     return PFalse;
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_GatekeeperReject>(this, H460_MessageType::e_gatekeeperReject, grj);
 #endif
 
@@ -558,7 +558,7 @@ void H225_RAS::OnSendRegistrationRequest(H323RasPDU & pdu, H225_RegistrationRequ
 {
   OnSendRegistrationRequest(rrq);
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   SendFeatureSet<H225_RegistrationRequest>(this, H460_MessageType::e_registrationRequest, rrq);
 #endif
 	
@@ -579,7 +579,7 @@ void H225_RAS::OnSendRegistrationConfirm(H323RasPDU & pdu, H225_RegistrationConf
     rcf.m_gatekeeperIdentifier = gatekeeperIdentifier;
   }
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   SendFeatureSet<H225_RegistrationConfirm>(this, H460_MessageType::e_registrationConfirm, rcf);
 #endif
 
@@ -604,7 +604,7 @@ void H225_RAS::OnSendRegistrationReject(H323RasPDU & pdu, H225_RegistrationRejec
 
   OnSendRegistrationReject(rrj);
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   SendFeatureSet<H225_RegistrationReject>(this, H460_MessageType::e_registrationReject, rrj);
 #endif
 
@@ -620,7 +620,7 @@ void H225_RAS::OnSendRegistrationReject(H225_RegistrationReject & /*rrj*/)
 
 PBoolean H225_RAS::OnReceiveRegistrationRequest(const H323RasPDU &, const H225_RegistrationRequest & rrq)
 {
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_RegistrationRequest>(this, H460_MessageType::e_registrationRequest, rrq);
 #endif
 	
@@ -653,7 +653,7 @@ PBoolean H225_RAS::OnReceiveRegistrationConfirm(const H323RasPDU & pdu, const H2
                          rcf.m_cryptoTokens, H225_RegistrationConfirm::e_cryptoTokens))
     return PFalse;
   
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_RegistrationConfirm>(this, H460_MessageType::e_registrationConfirm, rcf);
 #endif
 
@@ -677,7 +677,7 @@ PBoolean H225_RAS::OnReceiveRegistrationReject(const H323RasPDU & pdu, const H22
                          rrj.m_cryptoTokens, H225_RegistrationReject::e_cryptoTokens))
     return PFalse;
   
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_RegistrationReject>(this, H460_MessageType::e_registrationReject, rrj);
 #endif
 
@@ -791,7 +791,7 @@ void H225_RAS::OnSendAdmissionRequest(H323RasPDU & pdu, H225_AdmissionRequest &
 {
   OnSendAdmissionRequest(arq);
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   SendFeatureSet<H225_AdmissionRequest>(this, H460_MessageType::e_admissionRequest, arq);
 #endif
 	
@@ -809,7 +809,7 @@ void H225_RAS::OnSendAdmissionConfirm(H323RasPDU & pdu, H225_AdmissionConfirm &
 {
   OnSendAdmissionConfirm(acf);
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   SendFeatureSet<H225_AdmissionConfirm>(this, H460_MessageType::e_admissionConfirm, acf);
 #endif
 	
@@ -827,7 +827,7 @@ void H225_RAS::OnSendAdmissionReject(H323RasPDU & pdu, H225_AdmissionReject & ar
 {
   OnSendAdmissionReject(arj);
 	
-#ifdef OPAL_H460
+#if OPAL_H460
 	SendFeatureSet<H225_AdmissionReject>(this, H460_MessageType::e_admissionReject, arj);
 #endif
 	
@@ -848,7 +848,7 @@ PBoolean H225_RAS::OnReceiveAdmissionRequest(const H323RasPDU & pdu, const H225_
                          arq.m_cryptoTokens, H225_AdmissionRequest::e_cryptoTokens))
     return PFalse;
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_AdmissionRequest>(this, H460_MessageType::e_admissionRequest, arq);
 #endif
 
@@ -872,7 +872,7 @@ PBoolean H225_RAS::OnReceiveAdmissionConfirm(const H323RasPDU & pdu, const H225_
                          acf.m_cryptoTokens, H225_AdmissionConfirm::e_cryptoTokens))
     return PFalse;
   
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_AdmissionConfirm>(this, H460_MessageType::e_admissionConfirm, acf);
 #endif
 
@@ -896,7 +896,7 @@ PBoolean H225_RAS::OnReceiveAdmissionReject(const H323RasPDU & pdu, const H225_A
                          arj.m_cryptoTokens, H225_AdmissionReject::e_cryptoTokens))
     return PFalse;
   
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_AdmissionReject>(this, H460_MessageType::e_admissionReject, arj);
 #endif
 
@@ -1106,7 +1106,7 @@ void H225_RAS::OnSendLocationRequest(H323RasPDU & pdu, H225_LocationRequest & lr
 {
   OnSendLocationRequest(lrq);
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   SendFeatureSet<H225_LocationRequest>(this, H460_MessageType::e_locationRequest, lrq);
 #endif
 	
@@ -1127,7 +1127,7 @@ PBoolean H225_RAS::OnReceiveLocationRequest(const H323RasPDU & pdu, const H225_L
                          lrq.m_cryptoTokens, H225_LocationRequest::e_cryptoTokens))
     return PFalse;
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_LocationRequest>(this, H460_MessageType::e_locationRequest, lrq);
 #endif
 
@@ -1145,7 +1145,7 @@ void H225_RAS::OnSendLocationConfirm(H323RasPDU & pdu, H225_LocationConfirm & lc
 {
   OnSendLocationConfirm(lcf);
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   SendFeatureSet<H225_LocationConfirm>(this, H460_MessageType::e_locationConfirm, lcf);
 #endif
 	
@@ -1169,7 +1169,7 @@ PBoolean H225_RAS::OnReceiveLocationConfirm(const H323RasPDU &, const H225_Locat
     locatedAddress = lcf.m_callSignalAddress;
   }
   
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_LocationConfirm>(this, H460_MessageType::e_locationConfirm, lcf);
 #endif
 
@@ -1187,7 +1187,7 @@ void H225_RAS::OnSendLocationReject(H323RasPDU & pdu, H225_LocationReject & lrj)
 {
   OnSendLocationReject(lrj);
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   SendFeatureSet<H225_LocationReject>(this, H460_MessageType::e_locationReject, lrj);
 #endif
 	
@@ -1211,7 +1211,7 @@ PBoolean H225_RAS::OnReceiveLocationReject(const H323RasPDU & pdu, const H225_Lo
                          lrj.m_cryptoTokens, H225_LocationReject::e_cryptoTokens))
     return PFalse;
   
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_LocationReject>(this, H460_MessageType::e_locationReject, lrj);
 #endif
 
@@ -1292,7 +1292,7 @@ void H225_RAS::OnSendNonStandardMessage(H323RasPDU & pdu, H225_NonStandardMessag
 {
   OnSendNonStandardMessage(nsm);
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   SendFeatureSet<H225_NonStandardMessage>(this, H460_MessageType::e_nonStandardMessage, nsm);
 #endif
 	
@@ -1313,7 +1313,7 @@ PBoolean H225_RAS::OnReceiveNonStandardMessage(const H323RasPDU & pdu, const H22
                          nsm.m_cryptoTokens, H225_NonStandardMessage::e_cryptoTokens))
     return PFalse;
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_NonStandardMessage>(this, H460_MessageType::e_nonStandardMessage, nsm);
 #endif
 
@@ -1437,7 +1437,7 @@ void H225_RAS::OnSendServiceControlIndication(H323RasPDU & pdu, H225_ServiceCont
 {
   OnSendServiceControlIndication(sci);
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   SendFeatureSet<H225_ServiceControlIndication>(this, H460_MessageType::e_serviceControlIndication, sci);
 #endif
 	
@@ -1458,7 +1458,7 @@ PBoolean H225_RAS::OnReceiveServiceControlIndication(const H323RasPDU & pdu, con
                          sci.m_cryptoTokens, H225_ServiceControlIndication::e_cryptoTokens))
     return PFalse;
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_ServiceControlIndication>(this, H460_MessageType::e_serviceControlIndication, sci);
 #endif
 
@@ -1476,7 +1476,7 @@ void H225_RAS::OnSendServiceControlResponse(H323RasPDU & pdu, H225_ServiceContro
 {
   OnSendServiceControlResponse(scr);
 	
-#ifdef OPAL_H460
+#if OPAL_H460
   SendFeatureSet<H225_ServiceControlResponse>(this, H460_MessageType::e_serviceControlResponse, scr);
 #endif
 	
@@ -1500,7 +1500,7 @@ PBoolean H225_RAS::OnReceiveServiceControlResponse(const H323RasPDU & pdu, const
                          scr.m_cryptoTokens, H225_ServiceControlResponse::e_cryptoTokens))
     return PFalse;
   
-#ifdef OPAL_H460
+#if OPAL_H460
   ReceiveFeatureSet<H225_ServiceControlResponse>(this, H460_MessageType::e_serviceControlResponse, scr);
 #endif
 
