@@ -24,15 +24,15 @@
  *
  * The author of this code is Derek J Smithies
  *
- *
- *
- * $Revision: 20711 $
+ * $Revision: 21316 $
  * $Author: dereksmithies $
- * $Date: 2008-08-21 05:18:38 +0200 (jeu 21 aoû 2008) $
+ * $Date: 2008-10-13 21:35:09 +0000 (Mon, 13 Oct 2008) $
  */
 
 #include <ptlib.h>
-#include <typeinfo>
+#include <opal/buildopts.h>
+
+#if OPAL_IAX2
 
 #ifdef P_USE_PRAGMA
 #pragma implementation "callprocessor.h"
@@ -49,6 +49,8 @@
 #include <iax2/sound.h>
 #include <iax2/transmit.h>
 
+#include <typeinfo>
+
 #define new PNEW
 
 /********************************************************************************
@@ -250,7 +252,6 @@ void IAX2CallProcessor::ConnectToRemoteNode(PString & newRemoteNode)
   remote.SetRemoteAddress(ip);
     
   IAX2FullFrameProtocol * f = new IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdNew);
-  PTRACE(4, "Create full frame protocol to do cmdNew. Just contains data. ");
   f->AppendIe(new IAX2IeVersion());
   f->AppendIe(new IAX2IeFormat(con->GetPreferredCodec()));
   f->AppendIe(new IAX2IeCapability(con->GetSupportedCodecs()));
@@ -275,11 +276,10 @@ void IAX2CallProcessor::ConnectToRemoteNode(PString & newRemoteNode)
   if (!res[IAX2EndPoint::contextIndex].IsEmpty())
     f->AppendIe(new IAX2IeCalledContext(res[IAX2EndPoint::contextIndex]));
 
-#if P_SSL_AES
+#if OPAL_PTLIB_SSL_AES
   f->AppendIe(new IAX2IeEncryption());
 #endif
 
-  PTRACE(5, "Create full frame protocol to do cmdNew. Finished appending Ies. ");
   TransmitFrameToRemoteEndpoint(f);
   StartNoResponseTimer();
   return;
@@ -348,7 +348,9 @@ void IAX2CallProcessor::ProcessFullFrame(IAX2FullFrame & fullFrame)
 
 void IAX2CallProcessor::ProcessLists()
 {
-  while(ProcessOneIncomingEthernetFrame());
+  while (ProcessOneIncomingEthernetFrame()) {
+    ;
+  }
   
   PBYTEArray *oneSound;
   do {
@@ -907,12 +909,17 @@ void IAX2CallProcessor::ProcessIaxCmdNew(IAX2FullFrameProtocol *src)
   }
   
   SetCallNewed();
+  PTRACE(3, "ProcessIaxCmdNew have an incoming call to manage");
+  {
+    unsigned options = 0;
+    OpalConnection::StringOptions stringOptions;
+    con->OnIncomingConnection(options, &stringOptions);
+  }
+  con->OnSetUp();   //The person receiving call does ownerCall.OnSetUp();
 
-  con->OnSetUp();   //ONLY the  callee (person receiving call) does ownerCall.OnSetUp();
+  con->GetEndPoint().GetCodecLengths(selectedCodec, audioCompressedBytes, 
+				     audioFrameDuration);
 
-  con->GetEndPoint().GetCodecLengths(selectedCodec, audioCompressedBytes, audioFrameDuration);
-  PTRACE(4, "codec frame play duration is " << audioFrameDuration << " ms, which compressed to "
-         << audioCompressedBytes << " bytes of data");
 
   /*At this point, we have selected a codec to use. */
   reply = new  IAX2FullFrameProtocol(this, IAX2FullFrameProtocol::cmdAccept);
@@ -923,11 +930,10 @@ void IAX2CallProcessor::ProcessIaxCmdNew(IAX2FullFrameProtocol *src)
   /*We could send an AuthReq frame at this point */
 
   IAX2FullFrameSessionControl *r;
-  r = new IAX2FullFrameSessionControl(this, IAX2FullFrameSessionControl::ringing);
+  r = new IAX2FullFrameSessionControl(this, 
+				      IAX2FullFrameSessionControl::ringing);
   TransmitFrameToRemoteEndpoint(r, IAX2WaitingForAck::RingingAcked);
   delete src;
-
-  con->GetEndPoint().OnIncomingCall(*con);
 }
 
 void IAX2CallProcessor::ProcessIaxCmdAck(IAX2FullFrameProtocol *src)
@@ -1290,6 +1296,8 @@ PBoolean IAX2CallProcessor::IncomingMessageOutOfOrder(IAX2FullFrame *f)
 }
 
 
+#endif // OPAL_IAX2
+
 ////////////////////////////////////////////////////////////////////////////////
 
 /* The comment below is magic for those who use emacs to edit this file. */
