@@ -25,9 +25,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 21421 $
- * $Author: dereksmithies $
- * $Date: 2008-10-30 00:36:33 +0000 (Thu, 30 Oct 2008) $
+ * $Revision: 21986 $
+ * $Author: rjongbloed $
+ * $Date: 2009-02-04 04:31:07 +0000 (Wed, 04 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -506,12 +506,11 @@ void IAX2Connection::ReceivedSoundPacketFromNetwork(IAX2Frame *soundFrame)
       return;
     }
 
-    RTP_DataFrame *mediaFrame = new RTP_DataFrame();
+    RTP_DataFrame *mediaFrame = new RTP_DataFrame(soundFrame->GetMediaDataSize());
     mediaFrame->SetTimestamp(soundFrame->GetTimeStamp() * 8);
     mediaFrame->SetMarker(PFalse);
     mediaFrame->SetPayloadType(opalPayloadType);
 
-    mediaFrame->SetPayloadSize(soundFrame->GetMediaDataSize());
     mediaFrame->SetSize(mediaFrame->GetPayloadSize() + mediaFrame->GetHeaderSize());
     memcpy(mediaFrame->GetPayloadPtr(), soundFrame->GetMediaDataPointer(), 
 	   soundFrame->GetMediaDataSize());
