@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21119 $
+ * $Revision: 23028 $
  * $Author: rjongbloed $
- * $Date: 2008-09-22 05:27:30 +0000 (Mon, 22 Sep 2008) $
+ * $Date: 2009-07-02 10:39:41 +0000 (Thu, 02 Jul 2009) $
  */
 
 #include <ptlib.h>
@@ -354,21 +354,24 @@ OpalLine * OpalLineEndPoint::GetLine(const PString & lineName, bool enableAudio,
 {
   PWaitAndSignal mutex(linesMutex);
 
-  PTRACE(3, "LID EP\tGetLine " << lineName << ", enableAudio=" << enableAudio << ", terminating=" << terminating);
+  PTRACE(4, "LID EP\tGetLine " << lineName << ", enableAudio=" << enableAudio << ", terminating=" << terminating);
   for (OpalLineList::iterator line = lines.begin(); line != lines.end(); ++line) {
     PString lineToken = line->GetToken();
-
-    PTRACE(3, "LID EP\tGetLine name=\"" << lineToken << "\", enableAudio=" << line->IsAudioEnabled());
-
-    if ((lineName == defaultLine || lineToken == lineName) &&
-        (line->IsTerminal() == terminating) &&
-         line->IsPresent() &&
-        (!enableAudio || (!line->IsAudioEnabled() && line->EnableAudio()))) {
+    if (lineName != defaultLine && lineToken != lineName)
+      PTRACE(4, "LID EP\tNo match to line name=\"" << lineToken << "\", default=" << defaultLine);
+    else if (line->IsTerminal() == terminating)
+      PTRACE(4, "LID EP\tNo match to line name=\"" << lineToken << "\", terminating=" << line->IsTerminal());
+    else if (!line->IsPresent())
+      PTRACE(4, "LID EP\tNo match to line name=\"" << lineToken << "\", not present");
+    else if (enableAudio && (line->IsAudioEnabled() || !line->EnableAudio()))
+      PTRACE(4, "LID EP\tNo match to line name=\"" << lineToken << "\", enableAudio=" << line->IsAudioEnabled());
+    else {
       PTRACE(3, "LID EP\tGetLine found the line \"" << lineName << '"');
       return &*line;
     }  
   }
 
+  PTRACE(3, "LID EP\tGetLine could not find/enable \"" << lineName << '"');
   return NULL;
 }
 
@@ -464,6 +467,9 @@ OpalLineConnection::OpalLineConnection(OpalCall & call,
   remotePartyName = number;
   if (remotePartyName.IsEmpty())
     remotePartyName = "Unknown";
+  else
+    remotePartyAddress = remotePartyName + '@';
+  remotePartyAddress += GetToken();
 
   silenceDetector = new OpalLineSilenceDetector(line, (endpoint.GetManager().GetSilenceDetectParams()));
 
@@ -518,7 +524,7 @@ PBoolean OpalLineConnection::SetAlerting(const PString & /*calleeName*/, PBoolea
 {
   PTRACE(3, "LID Con\tSetAlerting " << *this);
 
-  if (GetPhase() != SetUpPhase) 
+  if (GetPhase() >= AlertingPhase) 
     return false;
 
   // switch phase 
@@ -581,7 +587,8 @@ OpalMediaStream * OpalLineConnection::CreateMediaStream(const OpalMediaFormat &
                                                         unsigned sessionID,
                                                         PBoolean isSource)
 {
-  if (sessionID != OpalMediaFormat::DefaultAudioSessionID)
+  OpalMediaFormatList formats = line.GetDevice().GetMediaFormats();
+  if (formats.FindFormat(mediaFormat) == formats.end())
     return OpalConnection::CreateMediaStream(mediaFormat, sessionID, isSource);
 
   return new OpalLineMediaStream(*this, mediaFormat, sessionID, isSource, line);
@@ -606,7 +613,7 @@ PBoolean OpalLineConnection::OnOpenMediaStream(OpalMediaStream & mediaStream)
 
 PBoolean OpalLineConnection::SetAudioVolume(PBoolean source, unsigned percentage)
 {
-  PSafePtr<OpalLineMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalLineMediaStream>(GetMediaStream(OpalMediaFormat::DefaultAudioSessionID, source));
+  PSafePtr<OpalLineMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalLineMediaStream>(GetMediaStream(OpalMediaType::Audio(), source));
   if (stream == NULL)
     return false;
 
@@ -617,7 +624,7 @@ PBoolean OpalLineConnection::SetAudioVolume(PBoolean source, unsigned percentage
 
 unsigned OpalLineConnection::GetAudioSignalLevel(PBoolean source)
 {
-  PSafePtr<OpalLineMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalLineMediaStream>(GetMediaStream(OpalMediaFormat::DefaultAudioSessionID, source));
+  PSafePtr<OpalLineMediaStream> stream = PSafePtrCast<OpalMediaStream, OpalLineMediaStream>(GetMediaStream(OpalMediaType::Audio(), source));
   if (stream == NULL)
     return UINT_MAX;
 
@@ -747,7 +754,7 @@ void OpalLineConnection::HandleIncoming(PThread &, INT)
       PThread::Sleep(100);
       if (GetPhase() >= ReleasingPhase)
         return;
-    } while (count <= minimumRingCount); // Wait till we have CLID
+    } while (count < minimumRingCount); // Wait till we have CLID
 
     // Get caller ID
     PString callerId;
@@ -759,7 +766,7 @@ void OpalLineConnection::HandleIncoming(PThread &, INT)
       else {
         PTRACE(3, "LID Con\tDetected Caller ID \"" << callerId << '"');
         remotePartyNumber = words[0].Trim();
-        remotePartyName = words[1].Trim();
+        remotePartyName = words[2].Trim();
         if (remotePartyName.IsEmpty())
           remotePartyName = remotePartyNumber;
       }
@@ -929,7 +936,7 @@ PBoolean OpalLineMediaStream::Open()
       return false;
   }
 
-  SetDataSize(GetDataSize());
+  SetDataSize(GetDataSize(), GetDataSize());
   PTRACE(3, "LineMedia\tStream opened for " << mediaFormat << ", using "
          << (notUsingRTP ? (useDeblocking ? "reblocked audio" : "audio frames") : "direct RTP"));
 
@@ -1098,13 +1105,13 @@ PBoolean OpalLineMediaStream::WriteData(const BYTE * buffer, PINDEX length, PIND
 }
 
 
-PBoolean OpalLineMediaStream::SetDataSize(PINDEX dataSize)
+PBoolean OpalLineMediaStream::SetDataSize(PINDEX /*dataSize*/, PINDEX frameSize)
 {
   if (notUsingRTP) {
     if (IsSource())
-      useDeblocking = !line.SetReadFrameSize(dataSize) || line.GetReadFrameSize() != dataSize;
+      useDeblocking = !line.SetReadFrameSize(frameSize) || line.GetReadFrameSize() != frameSize;
     else
-      useDeblocking = !line.SetWriteFrameSize(dataSize) || line.GetWriteFrameSize() != dataSize;
+      useDeblocking = !line.SetWriteFrameSize(frameSize) || line.GetWriteFrameSize() != frameSize;
 
     PTRACE(3, "LineMedia\tStream frame size: rd="
            << line.GetReadFrameSize() << " wr="
@@ -1112,7 +1119,8 @@ PBoolean OpalLineMediaStream::SetDataSize(PINDEX dataSize)
            << (useDeblocking ? "needs" : "no") << " reblocking.");
   }
 
-  return OpalMediaStream::SetDataSize(dataSize);
+  // Yes we set BOTH to frameSize, and ignore the dataSize parameter
+  return OpalMediaStream::SetDataSize(frameSize, frameSize);
 }
 
 
