@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21352 $
- * $Author: csoutheren $
- * $Date: 2008-10-16 00:31:22 +0000 (Thu, 16 Oct 2008) $
+ * $Revision: 22647 $
+ * $Author: rjongbloed $
+ * $Date: 2009-05-19 04:14:23 +0000 (Tue, 19 May 2009) $
  */
 
 #include <ptlib.h>
@@ -47,8 +47,9 @@
 #include <codec/vidcodec.h>
 #endif
 
-#include <h224/h224handler.h>
-#include <h224/h281handler.h>
+#if OPAL_HAS_H224
+#include <h224/h224.h>
+#endif
 
 #include <ptclib/random.h>
 
@@ -66,7 +67,11 @@ static const char * const DefaultMediaFormatOrder[] = {
   OPAL_GSM0610,
   OPAL_G728,
   OPAL_G711_ULAW_64K,
-  OPAL_G711_ALAW_64K
+  OPAL_G711_ALAW_64K,
+  OPAL_H264,
+  OPAL_MPEG4,
+  OPAL_H263,
+  OPAL_H261
 };
 
 #define new PNEW
@@ -157,11 +162,12 @@ OpalManager::OpalManager()
   , stun(NULL)
   , interfaceMonitor(NULL)
   , activeCalls(*this)
-  , clearingAllCalls(PFalse)
-#if OPAL_RTP_AGGREGATE
-  , useRTPAggregation(false)
+#ifdef OPAL_ZRTP
+  , zrtpEnabled(false)
 #endif
 {
+  m_recordManager = new OpalWAVRecordManager();
+
   rtpIpPorts.current = rtpIpPorts.base = 5000;
   rtpIpPorts.max = 5999;
 
@@ -169,7 +175,7 @@ OpalManager::OpalManager()
   tcpPorts.current = tcpPorts.base = tcpPorts.max = 0;
   udpPorts.current = udpPorts.base = udpPorts.max = 0;
 
-#if OPAL_PTLIB_VIDEO
+#if OPAL_VIDEO
   PStringArray devices = PVideoInputDevice::GetDriversDeviceNames("*"); // Get all devices on all drivers
   PINDEX i;
   for (i = 0; i < devices.GetSize(); ++i) {
@@ -178,19 +184,17 @@ OpalManager::OpalManager()
     videoInputDevice.deviceName = devices[i];
     break;
   }
-  autoStartTransmitVideo = !videoInputDevice.deviceName.IsEmpty();
+  SetAutoStartTransmitVideo(!videoInputDevice.deviceName.IsEmpty());
 
   devices = PVideoOutputDevice::GetDriversDeviceNames("*"); // Get all devices on all drivers
   for (i = 0; i < devices.GetSize(); ++i) {
     if ((devices[i] *= "*.yuv") || (devices[i] *= "null"))
       continue;
     videoOutputDevice.deviceName = devices[i];
+    videoPreviewDevice = videoOutputDevice;
     break;
   }
-  autoStartReceiveVideo = !videoOutputDevice.deviceName.IsEmpty();
-
-  if (autoStartReceiveVideo)
-    videoPreviewDevice = videoOutputDevice;
+  SetAutoStartReceiveVideo(!videoOutputDevice.deviceName.IsEmpty());
 #endif
 
   garbageCollector = PThread::Create(PCREATE_NOTIFIER(GarbageMain), "Opal Garbage");
@@ -217,6 +221,7 @@ OpalManager::~OpalManager()
   delete garbageCollector;
 
   delete stun;
+  delete m_recordManager;
   delete interfaceMonitor;
 
   PTRACE(4, "OpalMan\tDeleted manager.");
@@ -242,8 +247,7 @@ void OpalManager::ShutDownEndpoints()
   PTRACE(4, "OpalMan\tShutting down endpoints.");
 
   // Clear any pending calls, set flag so no calls can be received before endpoints removed
-  clearingAllCalls = true;
-  ClearAllCalls();
+  InternalClearAllCalls(OpalConnection::EndedByLocalUser, true, m_clearingAllCallsCount++ == 0);
 
   // Deregister the endpoints
   endpointsMutex.StartRead();
@@ -256,7 +260,7 @@ void OpalManager::ShutDownEndpoints()
   endpointList.RemoveAll();
   endpointsMutex.EndWrite();
 
-  clearingAllCalls = false; // Allow for endpoints to be added again.
+  --m_clearingAllCallsCount; // Allow for endpoints to be added again.
 }
 
 
@@ -347,14 +351,12 @@ PBoolean OpalManager::SetUpCall(const PString & partyA,
                             unsigned int options,
                             OpalConnection::StringOptions * stringOptions)
 {
-  if (clearingAllCalls) {
-    PTRACE(2, "OpalMan\tSet up call not performed as clearing all calls.");
-    return false;
-  }
-
   PTRACE(3, "OpalMan\tSet up call from " << partyA << " to " << partyB);
 
   OpalCall * call = CreateCall(userData);
+  if (call == NULL)
+    return false;
+
   token = call->GetToken();
 
   call->SetPartyB(partyB);
@@ -365,7 +367,7 @@ PBoolean OpalManager::SetUpCall(const PString & partyA,
   // thread.
   if (MakeConnection(*call, partyA, userData, options, stringOptions) && call->GetConnection(0)->SetUpConnection()) {
     PTRACE(3, "OpalMan\tSetUpCall succeeded, call=" << *call);
-    return PTrue;
+    return true;
   }
 
   PSafePtr<OpalConnection> connection = call->GetConnection(0);
@@ -374,7 +376,7 @@ PBoolean OpalManager::SetUpCall(const PString & partyA,
 
   token.MakeEmpty();
 
-  return PFalse;
+  return false;
 }
 
 
@@ -387,7 +389,7 @@ PBoolean OpalManager::IsCallEstablished(const PString & token)
 {
   PSafePtr<OpalCall> call = activeCalls.FindWithLock(token, PSafeReadOnly);
   if (call == NULL)
-    return PFalse;
+    return false;
 
   return call->IsEstablished();
 }
@@ -408,8 +410,10 @@ PBoolean OpalManager::ClearCall(const PString & token,
   {
     // Find the call by token, callid or conferenceid
     PSafePtr<OpalCall> call = activeCalls.FindWithLock(token, PSafeReference);
-    if (call == NULL)
-      return PFalse;
+    if (call == NULL) {
+      PTRACE(2, "OpalMan\tCould not find/lock call token \"" << token << '"');
+      return false;
+    }
 
     call->Clear(reason, sync);
   }
@@ -417,7 +421,7 @@ PBoolean OpalManager::ClearCall(const PString & token,
   if (sync != NULL)
     sync->Wait();
 
-  return PTrue;
+  return true;
 }
 
 
@@ -431,21 +435,33 @@ PBoolean OpalManager::ClearCallSynchronous(const PString & token,
 
 void OpalManager::ClearAllCalls(OpalConnection::CallEndReason reason, PBoolean wait)
 {
-  PTRACE(4, "OpalMan\tClearing all calls " << (wait ? " and waiting" : "asynchronously"));
+  InternalClearAllCalls(reason, wait, m_clearingAllCallsCount++ == 0);
+  --m_clearingAllCallsCount;
+}
 
-  bool oldFlag = clearingAllCalls;
-  clearingAllCalls = true;
 
-  // Remove all calls from the active list first
-  for (PSafePtr<OpalCall> call = activeCalls; call != NULL; ++call)
-    call->Clear(reason);
+void OpalManager::InternalClearAllCalls(OpalConnection::CallEndReason reason, bool wait, bool firstThread)
+{
+  PTRACE(3, "OpalMan\tClearing all calls " << (wait ? "and waiting" : "asynchronously")
+                      << ", " << (firstThread ? "primary" : "secondary") << " thread.");
 
-  if (wait) {
-    allCallsCleared.Wait();
-    PTRACE(4, "OpalMan\tAll calls cleared.");
+  if (firstThread) {
+    // Clear all the currentyl active calls
+    for (PSafePtr<OpalCall> call = activeCalls; call != NULL; ++call)
+      call->Clear(reason);
   }
 
-  clearingAllCalls = oldFlag;
+  if (wait) {
+    /* This is done this way as PSyncPoint only works for one thread at a time,
+       all subsequent threads will wait on the mutex for the first one to be
+       released from the PSyncPoint wait. */
+    m_clearingAllCallsMutex.Wait();
+    if (firstThread)
+      m_allCallsCleared.Wait();
+    m_clearingAllCallsMutex.Signal();
+  }
+
+  PTRACE(3, "OpalMan\tAll calls cleared.");
 }
 
 
@@ -457,8 +473,8 @@ void OpalManager::OnClearedCall(OpalCall & PTRACE_PARAM(call))
 
 OpalCall * OpalManager::InternalCreateCall()
 {
-  if (clearingAllCalls) {
-    PTRACE(2, "OpalMan\tCreate call not performed as clearing all calls.");
+  if (m_clearingAllCallsCount != 0) {
+    PTRACE(2, "OpalMan\tCreate call not performed as currently clearing all calls.");
     return NULL;
   }
 
@@ -488,12 +504,16 @@ PString OpalManager::GetNextCallToken()
   return psprintf("%c%08x%u", PRandom::Number('a', 'z'), PRandom::Number(), ++lastCallTokenID);
 }
 
-PBoolean OpalManager::MakeConnection(OpalCall & call, const PString & remoteParty, void * userData, unsigned int options, OpalConnection::StringOptions * stringOptions)
+PBoolean OpalManager::MakeConnection(OpalCall & call,
+                                     const PString & remoteParty,
+                                     void * userData,
+                                     unsigned int options,
+                                     OpalConnection::StringOptions * stringOptions)
 {
   PTRACE(3, "OpalMan\tSet up connection to \"" << remoteParty << '"');
 
   if (remoteParty.IsEmpty())
-    return PFalse;
+    return false;
 
   PCaselessString epname = remoteParty.Left(remoteParty.Find(':'));
 
@@ -509,34 +529,36 @@ PBoolean OpalManager::MakeConnection(OpalCall & call, const PString & remotePart
 
   if (ep != NULL) {
     if (ep->MakeConnection(call, remoteParty, userData, options, stringOptions))
-      return PTrue;
+      return true;
     PTRACE(1, "OpalMan\tCould not use endpoint for protocol \"" << epname << '"');
   }
   else {
     PTRACE(1, "OpalMan\tCould not find endpoint to handle protocol \"" << epname << '"');
   }
-  return PFalse;
+  return false;
 }
 
 PBoolean OpalManager::OnIncomingConnection(OpalConnection & /*connection*/)
 {
-  return PTrue;
+  return true;
 }
 
 PBoolean OpalManager::OnIncomingConnection(OpalConnection & /*connection*/, unsigned /*options*/)
 {
-  return PTrue;
+  return true;
 }
 
 PBoolean OpalManager::OnIncomingConnection(OpalConnection & connection, unsigned options, OpalConnection::StringOptions * stringOptions)
 {
-  PTRACE(3, "OpalMan\tOn incoming connection " << connection);
+  PTRACE(3, "OpalMan\tOnIncoming connection " << connection);
+
+  connection.OnApplyStringOptions();
 
   if (!OnIncomingConnection(connection))
-    return PFalse;
+    return false;
 
   if (!OnIncomingConnection(connection, options))
-    return PFalse;
+    return false;
 
   // See if we already have a B-Party in the call. If not, make one.
   if (connection.GetOtherPartyConnection() != NULL)
@@ -555,12 +577,20 @@ PBoolean OpalManager::OnIncomingConnection(OpalConnection & connection, unsigned
     }
   }
 
+  OpalConnection::StringOptions mergedOptions = connection.GetStringOptions();
+  if (stringOptions != NULL) {
+    for (PINDEX i = 0; i < stringOptions->GetSize(); ++i)
+      mergedOptions.SetAt(stringOptions->GetKeyAt(i), stringOptions->GetDataAt(i));
+  }
+
   // Use a routing algorithm to figure out who the B-Party is, and make second connection
-  return OnRouteConnection(connection.GetLocalPartyURL(), destination, call, options, stringOptions);
+  PStringSet routesTried;
+  return OnRouteConnection(routesTried, connection.GetLocalPartyURL(), destination, call, options, &mergedOptions);
 }
 
 
-bool OpalManager::OnRouteConnection(const PString & a_party,
+bool OpalManager::OnRouteConnection(PStringSet & routesTried,
+                                    const PString & a_party,
                                     const PString & b_party,
                                     OpalCall & call,
                                     unsigned options,
@@ -570,21 +600,37 @@ bool OpalManager::OnRouteConnection(const PString & a_party,
   for (;;) {
     PString route = ApplyRouteTable(a_party, b_party, tableEntry);
     if (route.IsEmpty()) {
+      // Check for if B-Party is an explicit address
+      if (FindEndPoint(b_party.Left(b_party.Find(':'))) != NULL)
+        return MakeConnection(call, b_party, NULL, options, stringOptions);
+
       PTRACE(3, "OpalMan\tCould not route a=\"" << a_party << "\", b=\"" << b_party << ", call=" << call);
       return false;
     }
 
+    // See if already tried, keep searching if this route has already failed
+    if (routesTried[route])
+      continue;
+    routesTried += route;
+
     // See if this route can be connected
     if (MakeConnection(call, route, NULL, options, stringOptions))
       return true;
 
     // Recursively call with translated route
-    if (route != b_party && OnRouteConnection(a_party, route, call, options, stringOptions))
+    if (OnRouteConnection(routesTried, a_party, route, call, options, stringOptions))
       return true;
   }
 }
 
 
+void OpalManager::OnProceeding(OpalConnection & connection)
+{
+  PTRACE(3, "OpalMan\tOnProceeding " << connection);
+
+  connection.GetCall().OnProceeding(connection);
+}
+
 void OpalManager::OnAlerting(OpalConnection & connection)
 {
   PTRACE(3, "OpalMan\tOnAlerting " << connection);
@@ -643,7 +689,7 @@ PBoolean OpalManager::OnForwarded(OpalConnection & PTRACE_PARAM(connection),
 			      const PString & /*forwardParty*/)
 {
   PTRACE(4, "OpalEP\tOnForwarded " << connection);
-  return PTrue;
+  return true;
 }
 
 
@@ -670,8 +716,7 @@ PBoolean OpalManager::OnOpenMediaStream(OpalConnection & PTRACE_PARAM(connection
                                         OpalMediaStream & PTRACE_PARAM(stream))
 {
   PTRACE(3, "OpalMan\tOnOpenMediaStream " << connection << ',' << stream);
-
-  return PTrue;
+  return true;
 }
 
 
@@ -708,9 +753,11 @@ PBoolean OpalManager::CreateVideoInputDevice(const OpalConnection & /*connection
   PVideoDevice::OpenArgs args = videoInputDevice;
   args.width = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameWidthOption(), PVideoFrameInfo::QCIFWidth);
   args.height = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameHeightOption(), PVideoFrameInfo::QCIFHeight);
-  args.rate = mediaFormat.GetClockRate()/mediaFormat.GetFrameTime();
+  unsigned maxRate = mediaFormat.GetClockRate()/mediaFormat.GetFrameTime();
+  if (args.rate > maxRate)
+    args.rate = maxRate;
 
-  autoDelete = PTrue;
+  autoDelete = true;
   device = PVideoInputDevice::CreateOpenedDevice(args, false);
   PTRACE_IF(2, device == NULL, "OpalCon\tCould not open video device \"" << args.deviceName << '"');
   return device != NULL;
@@ -728,7 +775,7 @@ PBoolean OpalManager::CreateVideoOutputDevice(const OpalConnection & connection,
       (videoPreviewDevice.driverName == "SDL" && videoOutputDevice.driverName == "SDL") ||
       (videoPreviewDevice.deviceName == "SDL" && videoOutputDevice.deviceName == "SDL")
       ))
-    return PFalse;
+    return false;
 
   // Make copy so we can adjust the size
   PVideoDevice::OpenArgs args = preview ? videoPreviewDevice : videoOutputDevice;
@@ -742,7 +789,7 @@ PBoolean OpalManager::CreateVideoOutputDevice(const OpalConnection & connection,
     args.deviceName.Splice(preview ? "Local Preview" : connection.GetRemotePartyName(), start, args.deviceName.Find('"', start)-start);
   }
 
-  autoDelete = PTrue;
+  autoDelete = true;
   device = PVideoOutputDevice::CreateOpenedDevice(args, false);
   return device != NULL;
 }
@@ -769,7 +816,7 @@ void OpalManager::DestroyMediaPatch(OpalMediaPatch * patch)
 
 PBoolean OpalManager::OnStartMediaPatch(const OpalMediaPatch & /*patch*/)
 {
-  return PTrue;
+  return true;
 }
 
 
@@ -795,9 +842,9 @@ PString OpalManager::ReadUserInput(OpalConnection & connection,
 {
   PTRACE(3, "OpalMan\tReadUserInput from " << connection);
 
-  connection.PromptUserInput(PTrue);
+  connection.PromptUserInput(true);
   PString digit = connection.GetUserInput(firstDigitTimeout);
-  connection.PromptUserInput(PFalse);
+  connection.PromptUserInput(false);
 
   if (digit.IsEmpty()) {
     PTRACE(2, "OpalMan\tReadUserInput first character timeout (" << firstDigitTimeout << " seconds) on " << *this);
@@ -857,21 +904,21 @@ void OpalManager::RouteEntry::PrintOn(ostream & strm) const
 PBoolean OpalManager::AddRouteEntry(const PString & spec)
 {
   if (spec[0] == '#') // Comment
-    return PFalse;
+    return false;
 
   if (spec[0] == '@') { // Load from file
     PTextFile file;
     if (!file.Open(spec.Mid(1), PFile::ReadOnly)) {
       PTRACE(1, "OpalMan\tCould not open route file \"" << file.GetFilePath() << '"');
-      return PFalse;
+      return false;
     }
     PTRACE(4, "OpalMan\tAdding routes from file \"" << file.GetFilePath() << '"');
-    PBoolean ok = PFalse;
+    PBoolean ok = false;
     PString line;
     while (file.good()) {
       file >> line;
       if (AddRouteEntry(line))
-        ok = PTrue;
+        ok = true;
     }
     return ok;
   }
@@ -879,34 +926,34 @@ PBoolean OpalManager::AddRouteEntry(const PString & spec)
   PINDEX equal = spec.Find('=');
   if (equal == P_MAX_INDEX) {
     PTRACE(2, "OpalMan\tInvalid route table entry: \"" << spec << '"');
-    return PFalse;
+    return false;
   }
 
   RouteEntry * entry = new RouteEntry(spec.Left(equal).Trim(), spec.Mid(equal+1).Trim());
   if (entry->regex.GetErrorCode() != PRegularExpression::NoError) {
     PTRACE(2, "OpalMan\tIllegal regular expression in route table entry: \"" << spec << '"');
     delete entry;
-    return PFalse;
+    return false;
   }
 
   PTRACE(4, "OpalMan\tAdded route \"" << *entry << '"');
   routeTableMutex.Wait();
   routeTable.Append(entry);
   routeTableMutex.Signal();
-  return PTrue;
+  return true;
 }
 
 
 PBoolean OpalManager::SetRouteTable(const PStringArray & specs)
 {
-  PBoolean ok = PFalse;
+  PBoolean ok = false;
 
   routeTableMutex.Wait();
   routeTable.RemoveAll();
 
   for (PINDEX i = 0; i < specs.GetSize(); i++) {
     if (AddRouteEntry(specs[i].Trim()))
-      ok = PTrue;
+      ok = true;
   }
 
   routeTableMutex.Signal();
@@ -1205,18 +1252,21 @@ PBoolean OpalManager::IsRTPNATEnabled(OpalConnection & /*conn*/,
 PBoolean OpalManager::TranslateIPAddress(PIPSocket::Address & localAddress,
                                      const PIPSocket::Address & remoteAddress)
 {
-  if (!translationAddress.IsValid())
-    return PFalse; // Have nothing to translate it to
-
   if (!IsLocalAddress(localAddress))
-    return PFalse; // Is already translated
+    return false; // Is already translated
 
   if (IsLocalAddress(remoteAddress))
-    return PFalse; // Does not need to be translated
+    return false; // Does not need to be translated
 
-  // Tranlsate it!
-  localAddress = translationAddress;
-  return PTrue;
+  if (translationAddress.IsValid()) {
+    localAddress = translationAddress; // Translate it!
+    return true;
+  }
+
+  if (stun != NULL && stun->GetNatType() != PSTUNClient::BlockedNat)
+    return stun->GetExternalAddress(localAddress); // Translate it!
+
+  return false; // Have nothing to translate it to
 }
 
 
@@ -1274,10 +1324,11 @@ PSTUNClient::NatTypes OpalManager::SetSTUNServer(const PString & server)
   }
 
   PSTUNClient::NatTypes type = stun->GetNatType();
+  PIPSocket::Address stunExternalAddress;
   if (type != PSTUNClient::BlockedNat)
-    stun->GetExternalAddress(translationAddress);
+    stun->GetExternalAddress(stunExternalAddress);
 
-  PTRACE(3, "OPAL\tSTUN server \"" << server << "\" replies " << type << ", external IP " << translationAddress);
+  PTRACE(3, "OPAL\tSTUN server \"" << server << "\" replies " << type << ", external IP " << stunExternalAddress);
 
   return type;
 }
@@ -1416,24 +1467,24 @@ static PBoolean SetVideoDevice(const PVideoDevice::OpenArgs & args, PVideoDevice
   if (pDevice != NULL) {
     delete pDevice;
     member = args;
-    return PTrue;
+    return true;
   }
 
   if (args.deviceName[0] != '#')
-    return PFalse;
+    return false;
 
   // Selected device by ordinal
   PStringArray devices = PVideoXxxDevice::GetDriversDeviceNames(args.driverName, args.pluginMgr);
   if (devices.IsEmpty())
-    return PFalse;
+    return false;
 
   PINDEX id = args.deviceName.Mid(1).AsUnsigned();
   if (id <= 0 || id > devices.GetSize())
-    return PFalse;
+    return false;
 
   member = args;
   member.deviceName = devices[id-1];
-  return PTrue;
+  return true;
 }
 
 
@@ -1459,10 +1510,10 @@ PBoolean OpalManager::SetVideoOutputDevice(const PVideoDevice::OpenArgs & args)
 PBoolean OpalManager::SetNoMediaTimeout(const PTimeInterval & newInterval) 
 {
   if (newInterval < 10)
-    return PFalse;
+    return false;
 
   noMediaTimeout = newInterval; 
-  return PTrue; 
+  return true; 
 }
 
 
@@ -1474,13 +1525,13 @@ void OpalManager::GarbageCollection()
 
   for (PList<OpalEndPoint>::iterator ep = endpointList.begin(); ep != endpointList.end(); ++ep) {
     if (!ep->GarbageCollection())
-      allCleared = PFalse;
+      allCleared = false;
   }
 
   endpointsMutex.EndRead();
 
-  if (allCleared && clearingAllCalls)
-    allCallsCleared.Signal();
+  if (allCleared && m_clearingAllCallsCount != 0)
+    m_allCallsCleared.Signal();
 }
 
 
@@ -1500,22 +1551,13 @@ void OpalManager::OnNewConnection(OpalConnection & /*conn*/)
 {
 }
 
-PBoolean OpalManager::UseRTPAggregation() const
-{ 
-#if OPAL_RTP_AGGREGATE
-  return useRTPAggregation; 
-#else
-  return PFalse;
-#endif
-}
-
-PBoolean OpalManager::StartRecording(const PString & callToken, const PFilePath & fn)
+bool OpalManager::StartRecording(const PString & callToken, const PFilePath & fn, bool mono)
 {
   PSafePtr<OpalCall> call = activeCalls.FindWithLock(callToken, PSafeReadWrite);
   if (call == NULL)
-    return PFalse;
+    return false;
 
-  return call->StartRecording(fn);
+  return call->StartRecording(fn, mono);
 }
 
 
@@ -1526,14 +1568,42 @@ bool OpalManager::IsRecording(const PString & callToken)
 }
 
 
-void OpalManager::StopRecording(const PString & callToken)
+bool OpalManager::StopRecording(const PString & callToken)
 {
   PSafePtr<OpalCall> call = activeCalls.FindWithLock(callToken, PSafeReadWrite);
-  if (call != NULL)
-    call->StopRecording();
+  if (call == NULL)
+    return false;
+
+  call->StopRecording();
+  return true;
 }
 
 
+void OpalManager::AddIMMediaFormats(OpalMediaFormatList & mediaFormats, const OpalConnection * /*connection*/) const
+{
+#if OPAL_HAS_MSRP
+  mediaFormats += OpalMSRP;
+#endif
+#if OPAL_HAS_SIPIM
+  mediaFormats += OpalSIPIM;
+#endif
+#if OPAL_HAS_RFC4103
+  mediaFormats += OpalT140;
+#endif
+#if OPAL_HAS_H224
+  mediaFormats += OpalH224AnnexQ;
+  mediaFormats += OpalH224Tunnelled;
+#endif
+}
+
+void OpalManager::OnApplyStringOptions(
+  OpalConnection & conn,
+  OpalConnection::StringOptions & stringOptions
+)
+{
+  conn.ApplyStringOptions(stringOptions);
+}
+
 /////////////////////////////////////////////////////////////////////////////
 
 OpalManager::InterfaceMonitor::InterfaceMonitor(OpalManager & manager)
@@ -1555,104 +1625,168 @@ void OpalManager::InterfaceMonitor::OnRemoveInterface(const PIPSocket::Interface
     stun->InvalidateCache();
 }
 
+#ifdef OPAL_ZRTP
+bool OpalManager::GetZRTPEnabled() const
+{
+  return zrtpEnabled;
+}
+#endif
+
 
 /////////////////////////////////////////////////////////////////////////////
 
-OpalRecordManager::Mixer_T::Mixer_T()
-  : OpalAudioMixer(PTrue)
+OpalWAVRecordManager::Mixer_T::Mixer_T()
+  :	OpalAudioMixer(true)
+  ,	m_mono(false)
+  ,	m_started(false)
+{
+}
+
+bool OpalWAVRecordManager::Mixer_T::Open(const PFilePath & fn, bool mono)
+{
+  PWaitAndSignal mut(mutex);
+
+  if (m_started) {
+    PTRACE(2, "OPAL\tCannot open mixer after it has started.");
+    return false;
+  }
+
+  m_file.SetFormat(OpalWAVFile::fmt_PCM);
+  if (!m_file.Open(fn, PFile::ReadWrite, PFile::Create|PFile::Truncate)) {
+    PTRACE(2, "OPAL\tCould not open file \"" << fn << '"');
+    return false;
+  }
+
+  m_mono = mono;
+  if (!mono)
+    m_file.SetChannels(2);
+
+  m_started = true;
+  PTRACE(4, "OPAL\t" << (mono ? "Mono" : "Stereo") << " mixer opened for file \"" << fn << '"');
+  return true;
+}
+
+bool OpalWAVRecordManager::Mixer_T::Close()
+{
+  RemoveAllStreams();
+
+  PWaitAndSignal mut(mutex);
+  m_started = false;
+  m_file.Close();
+
+  return true;
+}
+
+PBoolean OpalWAVRecordManager::Mixer_T::OnWriteAudio(const MixerFrame & mixerFrame)
+{
+  if (!m_file.IsOpen())
+    return false;
+
+  OpalAudioMixerStream::StreamFrame frame;
+  if (m_mono)
+    mixerFrame.GetMixedFrame(frame);
+  else
+    mixerFrame.GetStereoFrame(frame);
+  m_file.Write(frame.GetPointerAndLock(), frame.GetSize());
+  frame.Unlock();
+
+  return true;
+}
+
+
+//////////////////////////////////////////////////////////////////////////////
+
+OpalWAVRecordManager::OpalWAVRecordManager()
+{
+}
+
+
+OpalWAVRecordManager::~OpalWAVRecordManager()
+{
+  for (MixerMap_T::iterator mixer = m_mixers.begin(); mixer != m_mixers.end(); ++mixer)
+    delete mixer->second;
+}
+
+
+bool OpalWAVRecordManager::Open(const PString & callToken, const PFilePath & fn, bool mono)
+{
+  PWaitAndSignal mutex(m_mutex);
+
+  if (callToken.IsEmpty())
+    return false;
+
+  if (m_mixers.find(callToken) != m_mixers.end()) {
+    PTRACE(2, "OPAL\tAttempting to record for call " << callToken << " when already recording.");
+    return false;
+  }
+
+  Mixer_T * mixer = new Mixer_T;
+  if (mixer->Open(fn, mono)) {
+    m_mixers[callToken] = mixer;
+    PTRACE(4, "OPAL\tOpened recorder on call " << callToken);
+    return true;
+  }
+
+  delete mixer;
+  return false;
+}
+
+
+bool OpalWAVRecordManager::IsOpen(const PString & callToken) const
+{
+  PWaitAndSignal mutex(m_mutex);
+
+  MixerMap_T::const_iterator mixer = m_mixers.find(callToken);
+  if (mixer == m_mixers.end())
+    return false;
+
+  return mixer->second->IsOpen();
+}
+
+
+bool OpalWAVRecordManager::CloseStream(const PString & callToken, const std::string & streamId)
+{
+  PWaitAndSignal mutex(m_mutex);
+
+  MixerMap_T::iterator mixer = m_mixers.find(callToken);
+  if (mixer == m_mixers.end()) {
+    PTRACE(2, "OPAL\tAttempting to close stream on call " << callToken << " when not recording.");
+    return false;
+  }
+
+  mixer->second->RemoveStream(streamId);
+  PTRACE(4, "OPAL\tClosed stream " << streamId << " on call " << callToken);
+  return true;
+}
+
+
+bool OpalWAVRecordManager::Close(const PString & callToken)
+{
+  PWaitAndSignal mutex(m_mutex);
+
+  MixerMap_T::iterator mixer = m_mixers.find(callToken);
+  if (mixer == m_mixers.end()) {
+    PTRACE(2, "OPAL\tAttempting to close recording on call " << callToken << " when not recording.");
+    return false;
+  }
+
+  PTRACE(4, "OPAL\tClosed recorder on call " << callToken);
+  mixer->second->Close();
+  delete mixer->second;
+  m_mixers.erase(mixer);
+  return true;
+}
+
+
+bool OpalWAVRecordManager::WriteAudio(const PString & callToken, const std::string & strm, const RTP_DataFrame & rtp)
 { 
-  mono = PFalse; 
-  started = PFalse; 
-}
+  PWaitAndSignal mutex(m_mutex);
 
-PBoolean OpalRecordManager::Mixer_T::Open(const PFilePath & fn)
-{
-  PWaitAndSignal m(mutex);
-
-  if (!started) {
-    file.SetFormat(OpalWAVFile::fmt_PCM);
-    file.Open(fn, PFile::ReadWrite);
-    if (!mono)
-      file.SetChannels(2);
-    started = PTrue;
+  MixerMap_T::iterator mixer = m_mixers.find(callToken);
+  if (mixer == m_mixers.end()) {
+    PTRACE(4, "OPAL\tAttempt to write to call " << callToken << " when not recording.");
+    return false;
   }
-  return PTrue;
-}
 
-PBoolean OpalRecordManager::Mixer_T::Close()
-{
-  PWaitAndSignal m(mutex);
-  file.Close();
-  return PTrue;
-}
-
-PBoolean OpalRecordManager::Mixer_T::OnWriteAudio(const MixerFrame & mixerFrame)
-{
-  if (file.IsOpen()) {
-    OpalAudioMixerStream::StreamFrame frame;
-    if (mono) {
-      mixerFrame.GetMixedFrame(frame);
-      file.Write(frame.GetPointerAndLock(), frame.GetSize());
-      frame.Unlock();
-    } else {
-      mixerFrame.GetStereoFrame(frame);
-      file.Write(frame.GetPointerAndLock(), frame.GetSize());
-      frame.Unlock();
-    }
-  }
-  return PTrue;
-}
-
-OpalRecordManager::OpalRecordManager()
-{
-  started = PFalse;
-}
-
-PBoolean OpalRecordManager::Open(const PString & _callToken, const PFilePath & fn)
-{
-  PWaitAndSignal m(mutex);
-
-  if (_callToken.IsEmpty())
-    return PFalse;
-
-  if (token.IsEmpty())
-    token = _callToken;
-  else if (_callToken != token)
-    return PFalse;
-
-  return mixer.Open(fn);
-}
-
-PBoolean OpalRecordManager::CloseStream(const PString & _callToken, const std::string & _strm)
-{
-  {
-    PWaitAndSignal m(mutex);
-    if (_callToken.IsEmpty() || token.IsEmpty() || (token != _callToken))
-      return PFalse;
-
-    mixer.RemoveStream(_strm);
-  }
-  return PTrue;
-}
-
-PBoolean OpalRecordManager::Close(const PString & _callToken)
-{
-  {
-    PWaitAndSignal m(mutex);
-    if (_callToken.IsEmpty() || token.IsEmpty() || (token != _callToken))
-      return PFalse;
-
-    mixer.RemoveAllStreams();
-  }
-  mixer.Close();
-  return PTrue;
-}
-
-PBoolean OpalRecordManager::WriteAudio(const PString & _callToken, const std::string & strm, const RTP_DataFrame & rtp)
-{ 
-  PWaitAndSignal m(mutex);
-  if (_callToken.IsEmpty() || token.IsEmpty() || (token != _callToken))
-    return PFalse;
-
-  return mixer.Write(strm, rtp);
+  return mixer->second->Write(strm, rtp);
 }
