@@ -23,9 +23,9 @@
  *
  * Contributor(s): ________________________________________.
  *
- * $Revision: 22089 $
+ * $Revision: 23028 $
  * $Author: rjongbloed $
- * $Date: 2009-02-19 04:19:55 +0000 (Thu, 19 Feb 2009) $
+ * $Date: 2009-07-02 10:39:41 +0000 (Thu, 02 Jul 2009) $
  */
 
 #include <ptlib.h>
@@ -249,7 +249,7 @@ PBoolean OpalMediaStream::ReadPacket(RTP_DataFrame & packet)
 {
   unsigned oldTimestamp = timestamp;
 
-  PAssert(defaultDataSize >= (packet.GetSize() - RTP_DataFrame::MinHeaderSize), "buffer for media packet is too small");
+  PAssert(defaultDataSize <= (packet.GetSize() - packet.GetHeaderSize()), "default data size too large for packet");
 
   PINDEX lastReadCount;
   if (!ReadData(packet.GetPayloadPtr(), defaultDataSize, lastReadCount))
@@ -392,11 +392,15 @@ PBoolean OpalMediaStream::PushPacket(RTP_DataFrame & packet)
 }
 
 
-PBoolean OpalMediaStream::SetDataSize(PINDEX dataSize)
+PBoolean OpalMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameSize)
 {
   if (dataSize <= 0)
     return false;
 
+  // Quantise data size
+  if (frameSize > 1)
+    dataSize = (dataSize+frameSize-1)/frameSize * frameSize;
+
   PTRACE_IF(4, defaultDataSize != dataSize, "Media\tSet data size from " << defaultDataSize << " to " << dataSize);
   defaultDataSize = dataSize;
   return true;
@@ -433,11 +437,8 @@ PBoolean OpalMediaStream::SetPatch(OpalMediaPatch * patch)
   if (!safeLock.IsLocked())
     return false;
 
-  if (IsOpen()) {
-    mediaPatch = patch;
-    return true;
-  }
-  return false;
+  mediaPatch = patch;
+  return true;
 }
 
 
@@ -465,14 +466,14 @@ PBoolean OpalMediaStream::RemoveFilter(const PNotifier & Filter, const OpalMedia
 
 
 #if OPAL_STATISTICS
-void OpalMediaStream::GetStatistics(OpalMediaStatistics & statistics) const
+void OpalMediaStream::GetStatistics(OpalMediaStatistics & statistics, bool fromPatch) const
 {
   PSafeLockReadOnly safeLock(*this);
   if (!safeLock.IsLocked())
     return;
 
-  if (mediaPatch != NULL)
-    mediaPatch->GetStatistics(statistics);
+  if (mediaPatch != NULL && !fromPatch)
+    mediaPatch->GetStatistics(statistics, IsSink());
 }
 #endif
 
@@ -595,13 +596,12 @@ OpalRTPMediaStream::OpalRTPMediaStream(OpalRTPConnection & conn,
      whatever the RTP sender throws at us. For sink, we set it to the
      maximum size based on MTU (or other criteria). */
   defaultDataSize = isSource ? 2048 : conn.GetMaxRtpPayloadSize();
-
-  rtp.SetEncoding(mediaFormat.GetMediaType().GetDefinition()->GetRTPEncoding());
 }
 
 
 OpalRTPMediaStream::~OpalRTPMediaStream()
 {
+  Close();
 }
 
 
@@ -610,6 +610,7 @@ PBoolean OpalRTPMediaStream::Open()
   if (isOpen)
     return true;
 
+  rtpSession.SetEncoding(mediaFormat.GetMediaType().GetDefinition()->GetRTPEncoding());
   rtpSession.Reopen(IsSource());
 
   return OpalMediaStream::Open();
@@ -660,19 +661,12 @@ PBoolean OpalRTPMediaStream::WritePacket(RTP_DataFrame & packet)
   if (packet.GetPayloadSize() == 0)
     return true;
 
-  PBoolean ret;
-  ret = rtpSession.WriteData(packet);
-
-  return ret;
-
+  return rtpSession.WriteData(packet);
 }
 
 
-PBoolean OpalRTPMediaStream::SetDataSize(PINDEX dataSize)
+PBoolean OpalRTPMediaStream::SetDataSize(PINDEX PTRACE_PARAM(dataSize), PINDEX /*frameSize*/)
 {
-  if (dataSize <= 0)
-    return false;
-
   PTRACE(3, "Media\tRTP data size cannot be changed to " << dataSize << ", fixed at " << defaultDataSize);
   return true;
 }
@@ -694,23 +688,24 @@ void OpalRTPMediaStream::EnableJitterBuffer() const
 
 
 #if OPAL_STATISTICS
-void OpalRTPMediaStream::GetStatistics(OpalMediaStatistics & statistics) const
+void OpalRTPMediaStream::GetStatistics(OpalMediaStatistics & statistics, bool fromPatch) const
 {
   rtpSession.GetStatistics(statistics, IsSource());
-  OpalMediaStream::GetStatistics(statistics);
+  OpalMediaStream::GetStatistics(statistics, fromPatch);
 }
 #endif
 
 ///////////////////////////////////////////////////////////////////////////////
 
 OpalRawMediaStream::OpalRawMediaStream(OpalConnection & conn,
-                                     const OpalMediaFormat & mediaFormat,
+                                       const OpalMediaFormat & mediaFormat,
                                        unsigned sessionID,
                                        PBoolean isSource,
                                        PChannel * chan, PBoolean autoDelete)
   : OpalMediaStream(conn, mediaFormat, sessionID, isSource)
   , m_channel(chan)
   , m_autoDelete(autoDelete)
+  , m_silence(160) // At least 10ms
   , m_averageSignalSum(0)
   , m_averageSignalSamples(0)
 {
@@ -748,7 +743,10 @@ PBoolean OpalRawMediaStream::ReadData(BYTE * buffer, PINDEX size, PINDEX & lengt
     return false;
 
   length = m_channel->GetLastReadCount();
-  CollectAverage(buffer, length);
+
+  if (buffer != NULL)
+    CollectAverage(buffer, length);
+
   return true;
 }
 
@@ -815,7 +813,7 @@ bool OpalRawMediaStream::SetChannel(PChannel * chan, bool autoDelete)
   m_channel = chan;
   m_autoDelete = autoDelete;
 
-  SetDataSize(GetDataSize());
+  SetDataSize(GetDataSize(), sizeof(short));
 
   m_channelMutex.Signal();
 
@@ -828,13 +826,13 @@ bool OpalRawMediaStream::SetChannel(PChannel * chan, bool autoDelete)
 
 PBoolean OpalRawMediaStream::Close()
 {
-  if (!OpalMediaStream::Close())
+  if (!isOpen)
     return false;
 
   if (m_channel != NULL)
     m_channel->Close();
 
-  return true;
+  return OpalMediaStream::Close();
 }
 
 
@@ -957,22 +955,22 @@ OpalAudioMediaStream::OpalAudioMediaStream(OpalConnection & conn,
 }
 
 
-PBoolean OpalAudioMediaStream::SetDataSize(PINDEX dataSize)
+PBoolean OpalAudioMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameSize)
 {
   /* For efficiency reasons we will not accept a packet size that is too small.
      We move it up to the next even multiple, which has a danger of the remote not
      sending an even number of our multiplier. */
-  const unsigned MinTime = 10;
-  PINDEX minSize = mediaFormat.GetClockRate()/(1000/MinTime)*sizeof(short);
-  if (dataSize < minSize) {
-    PTRACE(1, "Media\tClamping audio stream data size from " << dataSize << " to minimum " << minSize);
-    dataSize = ((minSize+dataSize-1)/dataSize)*dataSize;
+  const unsigned MinTimeMilliseconds = 10;
+  PINDEX minSize = mediaFormat.GetClockRate()/(1000/MinTimeMilliseconds)*sizeof(short);
+  if (frameSize < minSize) {
+    PTRACE(1, "Media\tClamping audio stream frame size from " << frameSize << " to minimum " << minSize);
+    frameSize = minSize;
   }
 
   PTRACE(3, "Media\tAudio " << (IsSource() ? "source" : "sink") << " data size set to "
-         << dataSize << " bytes and " << soundChannelBuffers << " buffers.");
-  return OpalMediaStream::SetDataSize(dataSize) &&
-         ((PSoundChannel *)m_channel)->SetBuffers(dataSize, soundChannelBuffers);
+         << frameSize << " bytes and " << soundChannelBuffers << " buffers.");
+  return OpalMediaStream::SetDataSize(dataSize, frameSize) &&
+         ((PSoundChannel *)m_channel)->SetBuffers(frameSize, soundChannelBuffers);
 }
 
 
@@ -1005,6 +1003,7 @@ OpalVideoMediaStream::OpalVideoMediaStream(OpalConnection & conn,
 
 OpalVideoMediaStream::~OpalVideoMediaStream()
 {
+  Close();
   if (autoDelete) {
     delete inputDevice;
     delete outputDevice;
@@ -1012,7 +1011,7 @@ OpalVideoMediaStream::~OpalVideoMediaStream()
 }
 
 
-PBoolean OpalVideoMediaStream::SetDataSize(PINDEX dataSize)
+PBoolean OpalVideoMediaStream::SetDataSize(PINDEX dataSize, PINDEX frameSize)
 {
   if (inputDevice != NULL) {
     PINDEX minDataSize = inputDevice->GetMaxFrameBytes();
@@ -1025,7 +1024,7 @@ PBoolean OpalVideoMediaStream::SetDataSize(PINDEX dataSize)
       dataSize = minDataSize;
   }
 
-  return OpalMediaStream::SetDataSize(sizeof(PluginCodec_Video_FrameHeader) + dataSize); 
+  return OpalMediaStream::SetDataSize(sizeof(PluginCodec_Video_FrameHeader) + dataSize, frameSize);
 }
 
 
@@ -1064,7 +1063,7 @@ PBoolean OpalVideoMediaStream::Open()
     }
   }
 
-  SetDataSize(0); // Gets set to minimum of device buffer requirements
+  SetDataSize(1, 1); // Gets set to minimum of device buffer requirements
 
   return OpalMediaStream::Open();
 }
