@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21760 $
+ * $Revision: 23028 $
  * $Author: rjongbloed $
- * $Date: 2008-12-10 05:58:09 +0000 (Wed, 10 Dec 2008) $
+ * $Date: 2009-07-02 10:39:41 +0000 (Thu, 02 Jul 2009) $
  */
 
 #include <ptlib.h>
@@ -179,7 +179,7 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
   if (sink->primaryCodec != NULL) {
     PTRACE(4, "Patch\tCreated primary codec " << sourceFormat << "->" << destinationFormat << " with ID " << id);
 
-    if (!sinkStream->SetDataSize(sink->primaryCodec->GetOptimalDataFrameSize(false))) {
+    if (!sinkStream->SetDataSize(sink->primaryCodec->GetOptimalDataFrameSize(false), destinationFormat.GetFrameSize())) {
       PTRACE(1, "Patch\tSink stream " << *sinkStream << " cannot support data size "
               << sink->primaryCodec->GetOptimalDataFrameSize(PFalse));
       return false;
@@ -202,7 +202,7 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
 
     PTRACE(4, "Patch\tCreated two stage codec " << sourceFormat << "/" << intermediateFormat << "/" << destinationFormat << " with ID " << id);
 
-    if (!sinkStream->SetDataSize(sink->secondaryCodec->GetOptimalDataFrameSize(PFalse))) {
+    if (!sinkStream->SetDataSize(sink->secondaryCodec->GetOptimalDataFrameSize(false), destinationFormat.GetFrameSize())) {
       PTRACE(1, "Patch\tSink stream " << *sinkStream << " cannot support data size "
               << sink->secondaryCodec->GetOptimalDataFrameSize(PFalse));
       return false;
@@ -214,7 +214,7 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
            << " and " << *sink->secondaryCodec << ", data size=" << sinkStream->GetDataSize());
   }
 
-  source.SetDataSize(sink->primaryCodec->GetOptimalDataFrameSize(true));
+  source.SetDataSize(sink->primaryCodec->GetOptimalDataFrameSize(true), sourceFormat.GetFrameSize());
   return true;
 }
 
@@ -290,21 +290,28 @@ void OpalMediaPatch::UnLockSinkTranscoder() const
 
 
 #if OPAL_STATISTICS
-void OpalMediaPatch::GetStatistics(OpalMediaStatistics & statistics) const
+void OpalMediaPatch::GetStatistics(OpalMediaStatistics & statistics, bool fromSink) const
 {
   inUse.StartRead();
 
+  if (fromSink)
+    source.GetStatistics(statistics, true);
+
   if (!sinks.IsEmpty())
-    sinks.front().GetStatistics(statistics);
+    sinks.front().GetStatistics(statistics, !fromSink);
 
   inUse.EndRead();
 }
 
 
-void OpalMediaPatch::Sink::GetStatistics(OpalMediaStatistics & statistics) const
+void OpalMediaPatch::Sink::GetStatistics(OpalMediaStatistics & statistics, bool fromSource) const
 {
+  if (fromSource)
+    stream->GetStatistics(statistics, true);
+
   if (primaryCodec != NULL)
     primaryCodec->GetStatistics(statistics);
+
   if (secondaryCodec != NULL)
     secondaryCodec->GetStatistics(statistics);
 }
@@ -318,12 +325,9 @@ OpalMediaPatch::Sink::Sink(OpalMediaPatch & p, const OpalMediaStreamPtr & s)
   , secondaryCodec(NULL)
   , writeSuccessful(true)
 #if OPAL_VIDEO
-  , rcEnabled(false)
+  , rateController(NULL)
 #endif
 {
-  intermediateFrames.Append(new RTP_DataFrame);
-  finalFrames.Append(new RTP_DataFrame);
-
 #if OPAL_VIDEO
   SetRateControlParameters(stream->GetMediaFormat());
 #endif
@@ -336,6 +340,9 @@ OpalMediaPatch::Sink::~Sink()
 {
   delete primaryCodec;
   delete secondaryCodec;
+#if OPAL_VIDEO
+  delete rateController;
+#endif
 }
 
 
@@ -452,11 +459,23 @@ void OpalMediaPatch::Main()
 	
   bool asynchronous = OnPatchStart();
 
-  RTP_DataFrame sourceFrame(source.GetDataSize());
-  sourceFrame.SetPayloadType(source.GetMediaFormat().GetPayloadType());
+  /* Note the RTP frame is outside loop so that a) it is more efficient
+     for memory usage, the buffer is only ever increased and not allocated
+     on the heap ever time, and b) the timestamp value embedded into the
+     sourceFrame is needed for correct operation of the jitter buffer and
+     silence frames. It is adjusted by DispatchFrame (really Sink::WriteFrame)
+     each time and passed back in to source.Read() (and eventually the JB) so
+     it knows where it is up to in extracting data from the JB. */
+  RTP_DataFrame sourceFrame(0);
 
   while (source.IsOpen()) {
-    sourceFrame.SetPayloadSize(0); 
+    sourceFrame.SetPayloadType(source.GetMediaFormat().GetPayloadType());
+
+    // We do the following to make sure that the buffer size is large enough,
+    // in case something in previous loop adjusted it
+    sourceFrame.SetPayloadSize(source.GetDataSize());
+    sourceFrame.SetPayloadSize(0);
+
     if (!source.ReadPacket(sourceFrame)) {
       PTRACE(4, "Patch\tThread ended because source read failed");
       break;
@@ -570,8 +589,10 @@ static bool CannotTranscodeFrame(const OpalTranscoder & codec, RTP_DataFrame & f
     return true;
   }
 
-  if (!codec.AcceptEmptyPayload() && frame.GetPayloadSize() == 0) 
+  if (!codec.AcceptEmptyPayload() && frame.GetPayloadSize() == 0) {
+    frame.SetPayloadType(codec.GetPayloadType(false));
     return true;
+  }
 
   return false;
 }
@@ -580,24 +601,30 @@ static bool CannotTranscodeFrame(const OpalTranscoder & codec, RTP_DataFrame & f
 #if OPAL_VIDEO
 void OpalMediaPatch::Sink::SetRateControlParameters(const OpalMediaFormat & mediaFormat)
 {
-  unsigned targetBitRate = mediaFormat.GetOptionInteger(OpalVideoFormat::TargetBitRateOption());
-  rcEnabled = (mediaFormat.GetMediaType() == OpalMediaType::Video()) &&
-              mediaFormat != OpalYUV420P &&
-              mediaFormat.GetOptionBoolean(OpalVideoFormat::RateControlEnableOption());
+  if ((mediaFormat.GetMediaType() == OpalMediaType::Video()) && mediaFormat != OpalYUV420P) {
+    rateController = NULL;
+    PString rc = mediaFormat.GetOptionString(OpalVideoFormat::RateControllerOption());
+    if (rc.IsEmpty() && mediaFormat.GetOptionBoolean(OpalVideoFormat::RateControlEnableOption()))
+      rc = "Standard";
+    if (!rc.IsEmpty()) {
+      rateController = PFactory<OpalVideoRateController>::CreateInstance(rc);
+      if (rateController != NULL) {   
+        PTRACE(3, "Patch\tCreated " << rc << " rate controller");
+      }
+      else {
+        PTRACE(3, "Patch\tCould not create " << rc << " rate controller");
+      }
+    }
+  }
 
-  if (rcEnabled) 
-    rateController.Open(
-                        targetBitRate,
-                        mediaFormat.GetOptionInteger(OpalVideoFormat::FrameTimeOption(), -1),
-                        mediaFormat.GetOptionInteger(OpalVideoFormat::RateControlWindowSizeOption()),
-                        mediaFormat.GetOptionInteger(OpalVideoFormat::RateControlMaxFramesSkipOption())
-                        );
+  if (rateController != NULL) 
+    rateController->Open(mediaFormat);
 }
 
 
-bool OpalMediaPatch::Sink::RateControlExceeded()
+bool OpalMediaPatch::Sink::RateControlExceeded(bool & forceIFrame)
 {
-  if (!rcEnabled || !rateController.SkipFrame()) 
+  if ((rateController == NULL) || !rateController->SkipFrame(forceIFrame)) 
     return false;
 
   PTRACE(4, "Patch\tRate controller skipping frame.");
@@ -613,11 +640,29 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
     return false;
   
 #if OPAL_VIDEO
-  if (RateControlExceeded())
-    return true;
-
-  PInt64 totalPayloadSize = 0;
-  int frameCount = 0;
+  if (rateController != NULL) {
+    bool forceIFrame = false;
+    bool s = RateControlExceeded(forceIFrame);
+    if (forceIFrame)
+      stream->ExecuteCommand(OpalVideoUpdatePicture());
+    if (s) {
+      if (secondaryCodec == NULL) {
+        bool wasIFrame = false;
+        if (rateController->Pop(intermediateFrames, wasIFrame, false)) {
+        PTRACE(3, "RC returned " << intermediateFrames.GetSize() << " packets");
+          for (RTP_DataFrameList::iterator interFrame = intermediateFrames.begin(); interFrame != intermediateFrames.end(); ++interFrame) {
+            patch.FilterFrame(*interFrame, primaryCodec->GetOutputFormat());
+            if (!stream->WritePacket(*interFrame))
+              return (writeSuccessful = false);
+            sourceFrame.SetTimestamp(interFrame->GetTimestamp());
+            continue;
+          }
+          intermediateFrames.RemoveAll();
+        }
+      }
+      return true;
+    }
+  }
 #endif
 
   if (primaryCodec == NULL)
@@ -631,16 +676,31 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
     return false;
   }
 
+#if OPAL_VIDEO
+  if (secondaryCodec == NULL && rateController != NULL) {
+    PTRACE(4, "Patch\tPushing " << intermediateFrames.GetSize() << " packet into RC");
+    rateController->Push(intermediateFrames, ((OpalVideoTranscoder *)primaryCodec)->WasLastFrameIFrame());
+    bool wasIFrame = false;
+    if (rateController->Pop(intermediateFrames, wasIFrame, false)) {
+      PTRACE(4, "Patch\tPulled " << intermediateFrames.GetSize() << " frames from RC");
+      for (RTP_DataFrameList::iterator interFrame = intermediateFrames.begin(); interFrame != intermediateFrames.end(); ++interFrame) {
+        patch.FilterFrame(*interFrame, primaryCodec->GetOutputFormat());
+        if (!stream->WritePacket(*interFrame))
+          return (writeSuccessful = false);
+        sourceFrame.SetTimestamp(interFrame->GetTimestamp());
+        continue;
+      }
+      intermediateFrames.RemoveAll();
+    }
+  }
+  else 
+#endif
   for (RTP_DataFrameList::iterator interFrame = intermediateFrames.begin(); interFrame != intermediateFrames.end(); ++interFrame) {
     patch.FilterFrame(*interFrame, primaryCodec->GetOutputFormat());
 
     if (secondaryCodec == NULL) {
       if (!stream->WritePacket(*interFrame))
         return (writeSuccessful = false);
-#if OPAL_VIDEO
-      totalPayloadSize += interFrame->GetPayloadSize() + interFrame->GetHeaderSize();
-      frameCount++;
-#endif
       sourceFrame.SetTimestamp(interFrame->GetTimestamp());
       continue;
     }
@@ -660,17 +720,13 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
       patch.FilterFrame(*finalFrame, secondaryCodec->GetOutputFormat());
       if (!stream->WritePacket(*finalFrame))
         return (writeSuccessful = false);
-#if OPAL_VIDEO
-      totalPayloadSize += finalFrame->GetPayloadSize() + finalFrame->GetHeaderSize();
-      frameCount++;
-#endif
       sourceFrame.SetTimestamp(finalFrame->GetTimestamp());
     }
   }
 
 #if OPAL_VIDEO
-  if (rcEnabled)
-    rateController.AddFrame(totalPayloadSize, frameCount);
+  //if (rcEnabled)
+  //  rateController.AddFrame(totalPayloadSize, frameCount);
 #endif
 
   return true;
@@ -680,7 +736,7 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
 OpalMediaPatch::Thread::Thread(OpalMediaPatch & p)
   : PThread(65536,  //16*4kpage size
             NoAutoDeleteThread,
-            HighestPriority,
+            HighPriority,
             "Media Patch"),
     patch(p)
 {
@@ -704,8 +760,8 @@ void OpalPassiveMediaPatch::Start()
 
 PBoolean OpalPassiveMediaPatch::PushFrame(RTP_DataFrame & frame)
 {
-  DispatchFrame(frame);
-  return PTrue;
+  PReadWaitAndSignal mutex(inUse);
+  return DispatchFrame(frame);
 }
 
 
