@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22081 $
+ * $Revision: 22640 $
  * $Author: rjongbloed $
- * $Date: 2009-02-17 04:53:53 +0000 (Tue, 17 Feb 2009) $
+ * $Date: 2009-05-19 03:19:20 +0000 (Tue, 19 May 2009) $
  */
 
 #include <ptlib.h>
@@ -228,8 +228,8 @@ void OpalRTPConnection::OnMediaCommand(OpalMediaCommand & command, INT /*extra*/
         session->SendIntraFrameRequest();
 #if OPAL_STATISTICS
         m_VideoUpdateRequestsSent++;
-      }
 #endif
+      }
     }
   }
 }
@@ -312,6 +312,11 @@ OpalMediaStream * OpalRTPConnection::CreateMediaStream(const OpalMediaFormat & m
   if (ownerCall.IsMediaBypassPossible(*this, sessionID))
     return new OpalNullMediaStream(*this, mediaFormat, sessionID, isSource);
 
+  for (OpalMediaStreamPtr mediaStream(mediaStreams, PSafeReference); mediaStream != NULL; ++mediaStream) {
+    if (mediaStream->GetSessionID() == sessionID && mediaStream->IsSource() == isSource && !mediaStream->IsOpen())
+      return mediaStream;
+  }
+
   OpalMediaSession * mediaSession = GetMediaSession(sessionID);
   if (mediaSession == NULL) {
     PTRACE(1, "RTPCon\tCreateMediaStream could not find session " << sessionID);
@@ -343,10 +348,11 @@ void OpalRTPConnection::OnUserInputInlineRFC2833(OpalRFC2833Info & info, INT typ
     OnUserInputTone(info.GetTone(), info.GetDuration() > 0 ? info.GetDuration()/8 : 100);
 }
 
-void OpalRTPConnection::OnUserInputInlineCiscoNSE(OpalRFC2833Info & /*info*/, INT)
+void OpalRTPConnection::OnUserInputInlineCiscoNSE(OpalRFC2833Info & info, INT type)
 {
-  //if (!info.IsToneStart())
-  //  OnUserInputTone(info.GetTone(), info.GetDuration()/8);
+  // trigger on start of tone only
+  if (type == 0)
+    OnUserInputTone(info.GetTone(), info.GetDuration() > 0 ? info.GetDuration()/8 : 100);
 }
 
 void OpalRTPConnection::SessionFailing(RTP_Session & session)
