@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22519 $
+ * $Revision: 22928 $
  * $Author: rjongbloed $
- * $Date: 2009-05-06 05:04:07 +0000 (Wed, 06 May 2009) $
+ * $Date: 2009-06-19 05:26:12 +0000 (Fri, 19 Jun 2009) $
  */
 
 #include <ptlib.h>
@@ -258,6 +258,27 @@ bool SIPHandler::WriteSIPHandler(OpalTransport & transport)
 }
 
 
+bool SIPHandler::ActivateState(SIPHandler::State newState, unsigned msecs)
+{
+  PTimeInterval startTick = PTimer::Tick();
+  for (;;) {
+    {
+      PSafeLockReadWrite mutex(*this);
+      if (!mutex.IsLocked())
+        return false;
+
+      if (SendRequest(newState))
+        return true;
+    }
+
+    if ((PTimer::Tick() - startTick) > msecs)
+      return false;
+
+    PThread::Sleep(100);
+  }
+}
+
+
 PBoolean SIPHandler::SendRequest(SIPHandler::State newState)
 {
   expireTimer.Stop(false); // Stop automatic retry
@@ -268,14 +289,26 @@ PBoolean SIPHandler::SendRequest(SIPHandler::State newState)
   switch (newState) {
     case Unsubscribing:
       switch (state) {
-        case Unsubscribed :
-          return true;
-
         case Subscribed :
         case Unavailable :
-          break;
+          break;  // Can try and do Unsubscribe
 
-        default : // Are in the process of doing something
+        case Subscribing :
+        case Refreshing :
+        case Restoring :
+          PTRACE(2, "SIP\tCan't send " << newState << " request for " << GetMethod()
+                 << " handler while in " << state << " state, target="
+                 << GetAddressOfRecord() << ", id=" << GetCallID());
+          return false; // Are in the process of doing something
+
+        case Unsubscribed :
+        case Unsubscribing :
+          PTRACE(3, "SIP\tAlready doing " << state << " request for " << GetMethod()
+                 << " handler, target=" << GetAddressOfRecord() << ", id=" << GetCallID());
+          return true;  // Already done or doing it
+
+        default :
+          PAssertAlways(PInvalidParameter);
           return false;
       }
       break;
@@ -283,6 +316,29 @@ PBoolean SIPHandler::SendRequest(SIPHandler::State newState)
     case Subscribing :
     case Refreshing :
     case Restoring :
+      switch (state) {
+        case Subscribed :
+        case Unavailable :
+          break;  // Can do subscribe/refresh/restore
+
+        case Refreshing :
+        case Restoring :
+          PTRACE(3, "SIP\tAlready doing " << state << " request for " << GetMethod()
+                 << " handler, target=" << GetAddressOfRecord() << ", id=" << GetCallID());
+          return true; // Already doing it
+
+        case Subscribing :
+        case Unsubscribing :
+        case Unsubscribed :
+          PTRACE(2, "SIP\tCan't send " << newState << " request for " << GetMethod()
+                 << " handler while in " << state << " state, target="
+                 << GetAddressOfRecord() << ", id=" << GetCallID());
+          return false; // Can't restart as are on the way out
+
+        default : // Are in the process of doing something
+          PAssertAlways(PInvalidParameter);
+          return false;
+      }
       break;
 
     default :
@@ -397,7 +453,9 @@ void SIPHandler::OnReceivedAuthenticationRequired(SIPTransaction & transaction,
   CollapseFork(transaction);
 
   // Restart the transaction with new authentication handler
-  SendRequest(GetState());
+  State oldState = state;
+  state = Unavailable;
+  SendRequest(oldState);
 }
 
 
@@ -631,6 +689,7 @@ void SIPRegisterHandler::SendStatus(SIP_PDU::StatusCodes code, State state)
   status.m_addressofRecord = GetAddressOfRecord().AsString();
   status.m_productInfo = m_productInfo;
   status.m_reason = code;
+  status.m_userData = m_parameters.m_userData;
 
   switch (state) {
     case Subscribing :
@@ -714,8 +773,12 @@ SIPTransaction * SIPSubscribeHandler::CreateTransaction(OpalTransport &trans)
   // Default routeSet if there is a proxy
   m_dialog.UpdateRouteSet(m_proxy);
 
-  if (!m_dialog.IsEstablished())
-    m_dialog.SetLocalURI(endpoint.GetRegisteredPartyName(GetAddressOfRecord(), *m_transport));
+  if (!m_dialog.IsEstablished()) {
+    if (!m_parameters.m_from.IsEmpty())
+      m_dialog.SetLocalURI(m_parameters.m_from);
+    else
+      m_dialog.SetLocalURI(endpoint.GetRegisteredPartyName(GetAddressOfRecord(), *m_transport));
+  }
 
   m_parameters.m_expire = state != Unsubscribing ? expire : 0;
   return new SIPSubscribe(endpoint, trans, m_dialog, m_parameters);
@@ -856,7 +919,7 @@ PBoolean SIPSubscribeHandler::OnReceivedNOTIFY(SIP_PDU & request)
 
 class SIPMwiEventPackageHandler : public SIPEventPackageHandler
 {
-  virtual PString GetContentType() const
+  virtual PCaselessString GetContentType() const
   {
     return "application/simple-message-summary";
   }
@@ -910,7 +973,7 @@ static SIPEventPackageFactory::Worker<SIPMwiEventPackageHandler> mwiEventPackage
 
 class SIPPresenceEventPackageHandler : public SIPEventPackageHandler
 {
-  virtual PString GetContentType() const
+  virtual PCaselessString GetContentType() const
   {
     return "application/pidf+xml";
   }
@@ -920,8 +983,12 @@ class SIPPresenceEventPackageHandler : public SIPEventPackageHandler
     SIPURL from = request.GetMIME().GetFrom();
     from.Sanitise(SIPURL::ExternalURI);
 
+    SIPURL to = request.GetMIME().GetTo();
+    to.Sanitise(SIPURL::ExternalURI);
+
     SIPPresenceInfo info;
     info.m_address = from.AsQuotedString();
+    info.m_to      = to.AsQuotedString();
 
     // Check for empty body, if so then is OK, just a ping ...
     if (request.GetEntityBody().IsEmpty()) {
@@ -1021,7 +1088,7 @@ public:
   {
   }
 
-  virtual PString GetContentType() const
+  virtual PCaselessString GetContentType() const
   {
     return "application/dialog-info+xml";
   }
@@ -1283,17 +1350,22 @@ PBoolean SIPNotifyHandler::SendRequest(SIPHandler::State state)
 
 bool SIPNotifyHandler::SendNotify(const PObject * body)
 {
+  if (!LockReadWrite())
+    return false;
+
   if (m_packageHandler != NULL)
     SetBody(m_packageHandler->OnSendNOTIFY(*this, body));
   else if (body == NULL)
     SetBody(PString::Empty());
   else {
     PStringStream str;
-    str << body;
+    str << *body;
     SetBody(str);
   }
 
-  return SendRequest(Subscribing);
+  UnlockReadWrite();
+
+  return ActivateState(Subscribing, endpoint.GetNonInviteTimeout().GetInterval());
 }
 
 
@@ -1465,10 +1537,20 @@ SIPTransaction * SIPPingHandler::CreateTransaction(OpalTransport &t)
 
 //////////////////////////////////////////////////////////////////
 
+/* All of the bwlow search loops run through the list with only
+   PSafeReference rather than PSafeReadOnly, even though they are
+   reading fields from the handler instances. We can get away with
+   this becuase the information being tested, e.g. AOR, is constant
+   for the life of the handler instance, once constructed.
+
+   We need to use PSafeReference as there are some cases where
+   deadlocks can occur when locked handlers look for information
+   from other handlers.
+ */
 unsigned SIPHandlersList::GetCount(SIP_PDU::Methods meth, const PString & eventPackage) const
 {
   unsigned count = 0;
-  for (PSafePtr<SIPHandler> handler(*this, PSafeReference); handler != NULL; ++handler)
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler)
     if (handler->GetState () == SIPHandler::Subscribed &&
         handler->GetMethod() == meth &&
         (eventPackage.IsEmpty() || handler->GetEventPackage() == eventPackage))
@@ -1480,7 +1562,7 @@ unsigned SIPHandlersList::GetCount(SIP_PDU::Methods meth, const PString & eventP
 PStringList SIPHandlersList::GetAddresses(bool includeOffline, SIP_PDU::Methods meth, const PString & eventPackage) const
 {
   PStringList addresses;
-  for (PSafePtr<SIPHandler> handler(*this, PSafeReference); handler != NULL; ++handler)
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler)
     if ((includeOffline ? handler->GetState () != SIPHandler::Unsubscribed
                         : handler->GetState () == SIPHandler::Subscribed) &&
         handler->GetMethod() == meth &&
@@ -1493,11 +1575,12 @@ PStringList SIPHandlersList::GetAddresses(bool includeOffline, SIP_PDU::Methods
 /**
  * Find the SIPHandler object with the specified callID
  */
-PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByCallID(const PString & callID, PSafetyMode m)
+PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByCallID(const PString & callID, PSafetyMode mode)
 {
-  for (PSafePtr<SIPHandler> handler(*this, m); handler != NULL; ++handler)
-    if (callID == handler->GetCallID())
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+    if (callID == handler->GetCallID() && handler.SetSafetyMode(mode))
       return handler;
+  }
   return NULL;
 }
 
@@ -1505,18 +1588,44 @@ PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByCallID(const PString & cal
 /**
  * Find the SIPHandler object with the specified authRealm
  */
-PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByAuthRealm (const PString & authRealm, const PString & userName, PSafetyMode m)
+PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByAuthRealm (const PString & authRealm, const PString & userName, PSafetyMode mode)
 {
-  PIPSocket::Address realmAddress;
+  PIPSocket::Address handlerRealmAddress;
+  PIPSocket::Address authRealmAddress(authRealm);
 
-  for (PSafePtr<SIPHandler> handler(*this, m); handler != NULL; ++handler) {
-    if (authRealm == handler->GetRealm() && (userName.IsEmpty() || userName == handler->GetUsername()))
+  // if username is specified, look for exact matches
+  if (!userName.IsEmpty()) {
+
+    // look for a match to exact user name and realm
+    for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+      if ( handler->GetUsername() == userName &&
+          (handler->GetRealm().IsEmpty() || handler->GetRealm() == authRealm) &&
+           handler.SetSafetyMode(mode))
+        return handler;
+    }
+
+    // look for a match to exact username and realm as hostname
+    for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+      if (PIPSocket::GetHostAddress(handler->GetRealm(), handlerRealmAddress) &&
+          handlerRealmAddress == authRealmAddress &&
+          handler->GetUsername() == userName &&
+          handler.SetSafetyMode(mode))
+        return handler;
+    }
+  }
+
+  // look for a match to exact realm
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+    if (handler->GetRealm() == authRealm && handler.SetSafetyMode(mode))
       return handler;
   }
-  for (PSafePtr<SIPHandler> handler(*this, m); handler != NULL; ++handler) {
-    if (PIPSocket::GetHostAddress(handler->GetRealm(), realmAddress))
-      if (realmAddress == PIPSocket::Address(authRealm) && (userName.IsEmpty() || userName == handler->GetUsername()))
-        return handler;
+
+  // look for a match to exact realm as hostname
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+    if (PIPSocket::GetHostAddress(handler->GetRealm(), handlerRealmAddress) &&
+        handlerRealmAddress == authRealmAddress &&
+        handler.SetSafetyMode(mode))
+      return handler;
   }
   return NULL;
 }
@@ -1529,24 +1638,27 @@ PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByAuthRealm (const PString &
  * or 6001@seconix.com when registering 6001@seconix.com to
  * sip.seconix.com
  */
-PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByUrl(const PString & remoteAddress, SIP_PDU::Methods meth, PSafetyMode m)
+PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByUrl(const PString & remoteAddress, SIP_PDU::Methods meth, PSafetyMode mode)
 {
   SIPURL remoteURL = remoteAddress;
-  for (PSafePtr<SIPHandler> handler(*this, m); handler != NULL; ++handler) {
-    if (meth == handler->GetMethod() && remoteURL == handler->GetAddressOfRecord())
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+    if (handler->GetMethod() == meth &&
+        handler->GetAddressOfRecord() == remoteURL &&
+        handler.SetSafetyMode(mode))
       return handler;
   }
   return NULL;
 }
 
 
-PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByUrl(const PString & aor, SIP_PDU::Methods meth, const PString & eventPackage, PSafetyMode m)
+PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByUrl(const PString & aor, SIP_PDU::Methods meth, const PString & eventPackage, PSafetyMode mode)
 {
   SIPURL aorURL = aor;
-  for (PSafePtr<SIPHandler> handler(*this, m); handler != NULL; ++handler) {
-    if (meth == handler->GetMethod() &&
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+    if (handler->GetMethod() == meth &&
         handler->GetAddressOfRecord() == aorURL &&
-        handler->GetEventPackage() == eventPackage)
+        handler->GetEventPackage() == eventPackage &&
+        handler.SetSafetyMode(mode))
       return handler;
   }
   return NULL;
@@ -1558,14 +1670,14 @@ PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByUrl(const PString & aor, S
  * For example, in the above case, the name parameter
  * could be "sip.seconix.com" or "seconix.com".
  */
-PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByDomain(const PString & name, SIP_PDU::Methods meth, PSafetyMode m)
+PSafePtr<SIPHandler> SIPHandlersList::FindSIPHandlerByDomain(const PString & name, SIP_PDU::Methods meth, PSafetyMode mode)
 {
-  for (PSafePtr<SIPHandler> handler(*this, m); handler != NULL; ++handler) {
-
-    if ((handler->GetMethod() == meth) &&
-        (handler->GetState() != SIPHandler::Unsubscribed) &&
+  for (PSafePtr<SIPHandler> handler(m_handlersList, PSafeReference); handler != NULL; ++handler) {
+    if ( handler->GetMethod() == meth &&
+         handler->GetState() != SIPHandler::Unsubscribed &&
         (handler->GetAddressOfRecord().GetHostName() == name ||
-        handler->GetAddressOfRecord().GetHostAddress().IsEquivalent(name)))
+         handler->GetAddressOfRecord().GetHostAddress().IsEquivalent(name)) &&
+         handler.SetSafetyMode(mode))
       return handler;
   }
   return NULL;
