@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22184 $
+ * $Revision: 22995 $
  * $Author: rjongbloed $
- * $Date: 2009-03-10 02:45:29 +0000 (Tue, 10 Mar 2009) $
+ * $Date: 2009-06-26 06:23:20 +0000 (Fri, 26 Jun 2009) $
  */
 
 #include <ptlib.h>
@@ -123,7 +123,7 @@ void SIPEndPoint::ShutDown()
   bool shuttingDown = true;
   while (shuttingDown) {
     shuttingDown = false;
-    PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReference);
+    PSafePtr<SIPHandler> handler = activeSIPHandlers.GetFirstHandler();
     while (handler != NULL) {
       if (handler->ShutDown())
         activeSIPHandlers.Remove(handler++);
@@ -192,7 +192,7 @@ void SIPEndPoint::NATBindingRefresh(PTimer &, INT)
 
   if (natMethod != None) {
     PTRACE(5, "SIP\tNAT Binding refresh started.");
-    for (PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
+    for (PSafePtr<SIPHandler> handler = activeSIPHandlers.GetFirstHandler(PSafeReadOnly); handler != NULL; ++handler) {
 
       OpalTransport * transport = NULL;
       if (handler->GetState () != SIPHandler::Subscribed ||
@@ -318,14 +318,9 @@ static PString TranslateENUM(const PString & remoteParty)
   if (remoteParty.Find('@') == P_MAX_INDEX) {
 
     // make sure the number has only digits
-    PString e164 = remoteParty;
-    PINDEX pos = e164.Find(':');
-    if (pos != P_MAX_INDEX)
-      e164.Delete(0, pos+1);
-    pos = e164.FindSpan("0123456789*#", e164[0] != '+' ? 0 : 1);
-    e164.Delete(pos, P_MAX_INDEX);
-
-    if (!e164.IsEmpty()) {
+    PINDEX pos = remoteParty.Find(':');
+    PString e164 = pos != P_MAX_INDEX ? remoteParty.Mid(pos+1) : remoteParty;
+    if (e164.FindSpan("0123456789*#", e164[0] != '+' ? 0 : 1) == P_MAX_INDEX) {
       PString str;
       if (PDNS::ENUMLookup(e164, "E2U+SIP", str)) {
         PTRACE(4, "SIP\tENUM converted remote party " << remoteParty << " to " << str);
@@ -364,6 +359,8 @@ void SIPEndPoint::OnReleased(OpalConnection & connection)
 
 PBoolean SIPEndPoint::GarbageCollection()
 {
+  PTRACE(5, "SIP\tMONITOR:transactions=" << transactions.GetSize() << ",connections=" << connectionsActive.GetSize());
+
   PSafePtr<SIPTransaction> transaction(transactions, PSafeReadOnly);
   while (transaction != NULL) {
     if (transaction->IsTerminated()) {
@@ -378,7 +375,7 @@ PBoolean SIPEndPoint::GarbageCollection()
   bool transactionsDone = transactions.DeleteObjectsToBeRemoved();
 
 
-  PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReference);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.GetFirstHandler();
   while (handler != NULL) {
     // If unsubscribed then we do the shut down to clean up the handler
     if (handler->GetState() == SIPHandler::Unsubscribed && handler->ShutDown())
@@ -436,6 +433,7 @@ PBoolean SIPEndPoint::SetupTransfer(const PString & token,
   OpalConnection::StringOptions options;
   if (!callId.IsEmpty())
     options.SetAt(SIP_HEADER_REPLACES, callId);
+  options.SetAt(OPAL_OPT_CALLING_PARTY_URL, otherConnection->GetLocalPartyURL());
 
   SIPConnection * connection = CreateConnection(call, SIPURL::GenerateTag(), userData, TranslateENUM(remoteParty), NULL, NULL, 0, &options);
   if (!AddConnection(connection))
@@ -487,7 +485,7 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
     case SIP_PDU::Method_CANCEL :
       token = m_receivedConnectionTokens(mime.GetCallID());
       if (!token.IsEmpty()) {
-        threadPool.AddWork(new SIP_PDU_Work(*this, token, pdu));
+        m_connectionThreadPool.AddWork(new SIP_PDU_Work(*this, token, pdu), token);
         return true;
       }
       break;
@@ -496,7 +494,7 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
       if (toToken.IsEmpty()) {
         token = m_receivedConnectionTokens(mime.GetCallID());
         if (!token.IsEmpty()) {
-          threadPool.AddWork(new SIP_PDU_Work(*this, token, pdu));
+          m_connectionThreadPool.AddWork(new SIP_PDU_Work(*this, token, pdu), token);
           return true;
         }
 
@@ -517,22 +515,20 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
 
     case SIP_PDU::Method_ACK :
       pdu->AdjustVia(transport);
-      // Do next case
+      break;
 
     case SIP_PDU::NumMethods :
       break;
   }
 
-  if (hasFromConnection && hasToConnection)
-    token = pdu->GetMethod() != SIP_PDU::NumMethods ? fromToken : toToken;
+  if (hasToConnection)
+    token = toToken;
   else if (hasFromConnection)
     token = fromToken;
-  else if (hasToConnection)
-    token = toToken;
   else
     return OnReceivedConnectionlessPDU(transport, pdu);
 
-  threadPool.AddWork(new SIP_PDU_Work(*this, token, pdu));
+  m_connectionThreadPool.AddWork(new SIP_PDU_Work(*this, token, pdu), token);
   return true;
 }
 
@@ -540,9 +536,14 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
 bool SIPEndPoint::OnReceivedConnectionlessPDU(OpalTransport & transport, SIP_PDU * pdu)
 {
   if (pdu->GetMethod() == SIP_PDU::NumMethods || pdu->GetMethod() == SIP_PDU::Method_CANCEL) {
-    PSafePtr<SIPTransaction> transaction = GetTransaction(pdu->GetTransactionID(), PSafeReference);
-    if (transaction != NULL)
-      transaction->OnReceivedResponse(*pdu);
+    PString id;
+    if (activeSIPHandlers.FindSIPHandlerByCallID(id = pdu->GetMIME().GetCallID(), PSafeReference) != NULL ||
+        GetTransaction(id = pdu->GetTransactionID(), PSafeReference) != NULL) {
+      m_handlerThreadPool.AddWork(new SIP_PDU_Work(*this, id, pdu), id);
+      return true;
+    }
+
+    PTRACE(2, "SIP\tReceived response for unmatched transaction, id=" << id);
     return false;
   }
 
@@ -623,6 +624,7 @@ PBoolean SIPEndPoint::OnReceivedSUBSCRIBE(OpalTransport & transport, SIP_PDU & p
     dialog.SetRouteSet(mime.GetRecordRoute(true));
 
     handler = new SIPNotifyHandler(*this, dialog.GetRemoteURI().AsString(), eventPackage, dialog);
+    handler.SetSafetyMode(PSafeReadWrite);
     activeSIPHandlers.Append(handler);
 
     handler->GetTransport()->SetInterface(transport.GetInterface());
@@ -647,7 +649,7 @@ PBoolean SIPEndPoint::OnReceivedSUBSCRIBE(OpalTransport & transport, SIP_PDU & p
   if (expires > 0)
     handler->SendNotify(NULL);
   else
-    handler->SendRequest(SIPHandler::Unsubscribing);
+    handler->ActivateState(SIPHandler::Unsubscribing);
 
   return true;
 }
@@ -655,18 +657,6 @@ PBoolean SIPEndPoint::OnReceivedSUBSCRIBE(OpalTransport & transport, SIP_PDU & p
 
 void SIPEndPoint::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & response)
 {
-  PSafePtr<SIPHandler> handler = NULL;
- 
-  if (transaction.GetMethod() == SIP_PDU::Method_REGISTER
-      || transaction.GetMethod() == SIP_PDU::Method_SUBSCRIBE
-      || transaction.GetMethod() == SIP_PDU::Method_PUBLISH
-      || transaction.GetMethod() == SIP_PDU::Method_MESSAGE) {
-    // Have a response to various non-INVITE messages
-    handler = activeSIPHandlers.FindSIPHandlerByCallID(transaction.GetMIME().GetCallID(), PSafeReadWrite);
-    if (handler == NULL) 
-      return;
-  }
-
   switch (response.GetStatusCode()) {
     case SIP_PDU::Failure_IntervalTooBrief :
       OnReceivedIntervalTooBrief(transaction, response);
@@ -678,8 +668,7 @@ void SIPEndPoint::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & res
       break;
 
     case SIP_PDU::Failure_RequestTimeout :
-      if (handler != NULL)
-        handler->OnTransactionFailed(transaction);
+      OnTransactionFailed(transaction);
       break;
 
     default :
@@ -693,15 +682,72 @@ void SIPEndPoint::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & res
           break;
 
         default :
-          // Failure for a SUBSCRIBE/REGISTER/PUBLISH/MESSAGE 
-          if (handler != NULL)
+          PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(response.GetMIME().GetCallID(), PSafeReadWrite);
+          if (handler != NULL) 
             handler->OnFailed(response.GetStatusCode());
+          else {
+            PTRACE(2, "SIP\tResponse received for unknown handler ID: " << response.GetMIME().GetCallID());
+          }
           break;
       }
   }
 }
 
 
+PSafePtr<SIPConnection> SIPEndPoint::GetSIPConnectionWithLock(const PString & token,
+                                                              PSafetyMode mode,
+                                                              SIP_PDU::StatusCodes * errorCode)
+{
+  PSafePtr<SIPConnection> connection = PSafePtrCast<OpalConnection, SIPConnection>(GetConnectionWithLock(token, mode));
+  if (connection != NULL)
+    return connection;
+
+  PString to;
+  static const char toTag[] = ";to-tag=";
+  PINDEX pos = token.Find(toTag);
+  if (pos != P_MAX_INDEX) {
+    pos += sizeof(toTag)-1;
+    to = token(pos, token.Find(';', pos)-1).Trim();
+  }
+
+  PString from;
+  static const char fromTag[] = ";from-tag=";
+  pos = token.Find(fromTag);
+  if (pos != P_MAX_INDEX) {
+    pos += sizeof(fromTag)-1;
+    from = token(pos, token.Find(';', pos)-1).Trim();
+  }
+
+  PString callid = token.Left(token.Find(';')).Trim();
+  if (callid.IsEmpty() || to.IsEmpty() || from.IsEmpty()) {
+    if (errorCode != NULL)
+      *errorCode = SIP_PDU::Failure_BadRequest;
+    return NULL;
+  }
+
+  connection = PSafePtrCast<OpalConnection, SIPConnection>(connectionsActive.GetAt(0, PSafeReference));
+  while (connection != NULL) {
+    const SIPDialogContext & context = connection->GetDialog();
+    if (context.GetCallID() == callid) {
+      if (context.GetLocalTag() == to && context.GetRemoteTag() == from) {
+        if (connection.SetSafetyMode(mode))
+          return connection;
+        break;
+      }
+
+      PTRACE(4, "SIP\tReplaces header matches callid, but not to/from tags: "
+                "to=" << context.GetLocalTag() << ", from=" << context.GetRemoteTag());
+    }
+
+    ++connection;
+  }
+
+  if (errorCode != NULL)
+    *errorCode = SIP_PDU::Failure_TransactionDoesNotExist;
+  return NULL;
+}
+
+
 PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * request)
 {
   SIPMIMEInfo & mime = request->GetMIME();
@@ -709,46 +755,35 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
   // parse the incoming To field, and check if we accept incoming calls for this address
   SIPURL toAddr(mime.GetTo());
   if (!IsAcceptedAddress(toAddr)) {
-    PTRACE(2, "SIP\tIncoming INVITE from " << request->GetURI() << " for unacceptable address " << toAddr);
+    PTRACE(2, "SIP\tIncoming INVITE for " << request->GetURI() << " for unacceptable address " << toAddr);
     request->SendResponse(transport, SIP_PDU::Failure_NotFound, this);
-    return PFalse;
+    return false;
+  }
+
+  if (!request->GetEntityBody().IsEmpty() &&
+         (!mime.GetContentEncoding().IsEmpty() ||
+           mime.GetContentType() != "application/sdp")) {
+    // Do not currently support anything other than SDP, in particular multipart stuff.
+    PTRACE(2, "SIP\tIncoming INVITE for " << request->GetURI() << " does not contain SDP");
+    SIP_PDU response(*request, SIP_PDU::Failure_UnsupportedMediaType);
+    response.GetMIME().SetAccept("application/sdp");
+    response.GetMIME().SetAcceptEncoding("identity");
+    response.SetAllow(GetAllowedMethods());
+    request->SendResponse(transport, response, this);
+    return false;
   }
 
   // See if we are replacing an existing call.
   OpalCall * call = NULL;
   if (mime.Contains("Replaces")) {
-    PString replaces = mime("Replaces");
-
-    PString to;
-    static const char toTag[] = ";to-tag=";
-    PINDEX pos = replaces.Find(toTag);
-    if (pos != P_MAX_INDEX) {
-      pos += sizeof(toTag)-1;
-      to = replaces(pos, replaces.Find(';', pos)-1).Trim();
-    }
-
-    PString from;
-    static const char fromTag[] = ";from-tag=";
-    pos = replaces.Find(fromTag);
-    if (pos != P_MAX_INDEX) {
-      pos += sizeof(fromTag)-1;
-      from = replaces(pos, replaces.Find(';', pos)-1).Trim();
-    }
-
-    PString callid = replaces.Left(replaces.Find(';')).Trim();
-    if (callid.IsEmpty() || to.IsEmpty() || from.IsEmpty()) {
-      PTRACE(2, "SIP\tBad Replaces header in INVITE from " << request->GetURI());
-      request->SendResponse(transport, SIP_PDU::Failure_BadRequest, this);
-      return false;
-    }
-
-    PSafePtr<SIPConnection> replacedConnection = GetSIPConnectionWithLock(callid, PSafeReadOnly);
-    if (replacedConnection == NULL ||
-        replacedConnection->GetDialog().GetLocalTag() != to ||
-        replacedConnection->GetDialog().GetRemoteTag() != from) {
-      PTRACE(2, "SIP\tUnknown " << (replacedConnection != NULL ? "to/from" : "call-id")
-             << " in Replaces header of INVITE from " << request->GetURI());
-      request->SendResponse(transport, SIP_PDU::Failure_TransactionDoesNotExist, this);
+    SIP_PDU::StatusCodes errorCode;
+    PSafePtr<SIPConnection> replacedConnection = GetSIPConnectionWithLock(mime("Replaces"), PSafeReference, &errorCode);
+    if (replacedConnection == NULL) {
+      PTRACE_IF(2, errorCode==SIP_PDU::Failure_BadRequest,
+                "SIP\tBad Replaces header in INVITE for " << request->GetURI());
+      PTRACE_IF(2, errorCode==SIP_PDU::Failure_TransactionDoesNotExist,
+                "SIP\tNo connection matching dialog info in Replaces header of INVITE from " << request->GetURI());
+      request->SendResponse(transport, errorCode, this);
       return false;
     }
 
@@ -764,7 +799,7 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
   else {
     newTransport = CreateTransport(SIPURL("", transport.GetRemoteAddress(), 0), transport.GetInterface());
     if (newTransport == NULL) {
-      PTRACE(1, "SIP\tFailed to create transport for SIPConnection for INVITE from " << request->GetURI() << " for " << toAddr);
+      PTRACE(1, "SIP\tFailed to create transport for SIPConnection for INVITE for " << request->GetURI() << " to " << toAddr);
       request->SendResponse(transport, SIP_PDU::Failure_NotFound, this);
       return PFalse;
     }
@@ -787,15 +822,16 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
                                                newTransport,
                                                request);
   if (!AddConnection(connection)) {
-    PTRACE(1, "SIP\tFailed to create SIPConnection for INVITE from " << request->GetURI() << " for " << toAddr);
+    PTRACE(1, "SIP\tFailed to create SIPConnection for INVITE for " << request->GetURI() << " to " << toAddr);
     request->SendResponse(transport, SIP_PDU::Failure_NotFound, this);
     return PFalse;
   }
 
-  m_receivedConnectionTokens.SetAt(mime.GetCallID(), connection->GetToken());
+  PString token = connection->GetToken();
+  m_receivedConnectionTokens.SetAt(mime.GetCallID(), token);
 
   // Get the connection to handle the rest of the INVITE in the thread pool
-  threadPool.AddWork(new SIP_PDU_Work(*this, connection->GetToken(), request));
+  m_connectionThreadPool.AddWork(new SIP_PDU_Work(*this, token, request), token);
 
   return PTrue;
 }
@@ -803,9 +839,11 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
 void SIPEndPoint::OnReceivedIntervalTooBrief(SIPTransaction & transaction, SIP_PDU & response)
 {
   const SIPMIMEInfo & responseMIME = response.GetMIME();
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(responseMIME.GetCallID(), PSafeReadOnly);
-  if (handler == NULL)
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(responseMIME.GetCallID(), PSafeReadWrite);
+  if (handler == NULL) {
+    PTRACE(2, "SIP\tInterval too brief received for unknown handler ID: " << response.GetMIME().GetCallID());
     return;
+  }
 
   SIPTransaction *newTransaction = handler->CreateTransaction(transaction.GetTransport());
   if (newTransaction) {
@@ -818,37 +856,39 @@ void SIPEndPoint::OnReceivedIntervalTooBrief(SIPTransaction & transaction, SIP_P
 
   PTRACE(1, "SIP\t Could not restart REGISTER after IntervalTooBrief error!");
   handler->OnFailed(SIP_PDU::Failure_IntervalTooBrief);
-  return;
 }
 
 void SIPEndPoint::OnReceivedAuthenticationRequired(SIPTransaction & transaction, SIP_PDU & response)
 {
   // Try to find authentication information for the given call ID
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(response.GetMIME().GetCallID(), PSafeReadOnly);
-  if (handler == NULL)
-    return;
-  
-  handler->OnReceivedAuthenticationRequired(transaction, response);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(response.GetMIME().GetCallID(), PSafeReadWrite);
+  if (handler != NULL)
+    handler->OnReceivedAuthenticationRequired(transaction, response);
+  else {
+    PTRACE(2, "SIP\tAuthentication required received for unknown handler ID: " << response.GetMIME().GetCallID());
+  }
 }
 
 
 void SIPEndPoint::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response)
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(response.GetMIME().GetCallID(), PSafeReadOnly);
-  if (handler == NULL) 
-    return;
-  
-  handler->OnReceivedOK(transaction, response);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(response.GetMIME().GetCallID(), PSafeReadWrite);
+  if (handler != NULL) 
+    handler->OnReceivedOK(transaction, response);
+  else {
+    PTRACE(2, "SIP\tOK received for unknown handler ID: " << response.GetMIME().GetCallID());
+  }
 }
     
 
 void SIPEndPoint::OnTransactionFailed(SIPTransaction & transaction)
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(transaction.GetMIME().GetCallID(), PSafeReadOnly);
-  if (handler == NULL) 
-    return;
-  
-  handler->OnTransactionFailed(transaction);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(transaction.GetMIME().GetCallID(), PSafeReadWrite);
+  if (handler != NULL) 
+    handler->OnTransactionFailed(transaction);
+  else {
+    PTRACE(2, "SIP\tResponse received for unknown handler ID: " << transaction.GetMIME().GetCallID());
+  }
 }
 
 
@@ -860,14 +900,14 @@ PBoolean SIPEndPoint::OnReceivedNOTIFY(OpalTransport & transport, SIP_PDU & pdu)
   
   // A NOTIFY will have the same CallID than the SUBSCRIBE request it corresponds to
   // Technically should check for whole dialog, but call-id will do.
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(pdu.GetMIME().GetCallID(), PSafeReadOnly);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(pdu.GetMIME().GetCallID(), PSafeReadWrite);
 
   if (handler == NULL && eventPackage == SIPSubscribe::MessageSummary) {
     PTRACE(4, "SIP\tWork around Asterisk bug in message-summary event package.");
     SIPURL url_from (pdu.GetMIME().GetFrom());
     SIPURL url_to (pdu.GetMIME().GetTo());
     PString to = url_to.GetUserName() + "@" + url_from.GetHostName();
-    handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_SUBSCRIBE, eventPackage, PSafeReadOnly);
+    handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_SUBSCRIBE, eventPackage, PSafeReadWrite);
   }
 
   if (handler == NULL) {
@@ -1030,13 +1070,12 @@ bool SIPEndPoint::Register(const SIPRegister::Params & params, PString & aor)
   else {
     // Otherwise create a new request with this method type
     handler = CreateRegisterHandler(params);
-    handler.SetSafetyMode(PSafeReadWrite);
     activeSIPHandlers.Append(handler);
   }
 
   aor = handler->GetAddressOfRecord().AsString();
 
-  if (handler->SendRequest(SIPHandler::Subscribing))
+  if (handler->ActivateState(SIPHandler::Subscribing))
     return true;
 
   activeSIPHandlers.Remove(handler);
@@ -1052,28 +1091,26 @@ SIPRegisterHandler * SIPEndPoint::CreateRegisterHandler(const SIPRegister::Param
 
 PBoolean SIPEndPoint::Unregister(const PString & addressOfRecord)
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(addressOfRecord, SIP_PDU::Method_REGISTER, PSafeReadOnly);
-  if (handler == NULL) {
-    PTRACE(1, "SIP\tCould not find active REGISTER for " << addressOfRecord);
-    return false;
-  }
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(addressOfRecord, SIP_PDU::Method_REGISTER, PSafeReference);
+  if (handler != NULL)
+    return handler->ActivateState(SIPHandler::Unsubscribing);
 
-  return handler->SendRequest(SIPHandler::Unsubscribing);
+  PTRACE(1, "SIP\tCould not find active REGISTER for " << addressOfRecord);
+  return false;
 }
 
 
 bool SIPEndPoint::UnregisterAll()
 {
-  bool ok = true;
+  bool atLeastOne = false;
 
-  for (PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
-    if (handler->GetMethod() == SIP_PDU::Method_REGISTER) {
-      if (!handler->SendRequest(SIPHandler::Unsubscribing))
-        ok = false;
-    }
+  for (PSafePtr<SIPHandler> handler = activeSIPHandlers.GetFirstHandler(); handler != NULL; ++handler) {
+    if (handler->GetMethod() == SIP_PDU::Method_REGISTER &&
+        handler->ActivateState(SIPHandler::Unsubscribing))
+      atLeastOne = true;
   }
 
-  return ok;
+  return atLeastOne;
 }
 
 
@@ -1098,7 +1135,7 @@ bool SIPEndPoint::Subscribe(const SIPSubscribe::Params & params, PString & aor)
 
   // Create the SIPHandler structure
   PSafePtr<SIPSubscribeHandler> handler = PSafePtrCast<SIPHandler, SIPSubscribeHandler>(
-          activeSIPHandlers.FindSIPHandlerByUrl(params.m_addressOfRecord, SIP_PDU::Method_SUBSCRIBE, params.m_eventPackage, PSafeReadOnly));
+          activeSIPHandlers.FindSIPHandlerByUrl(params.m_addressOfRecord, SIP_PDU::Method_SUBSCRIBE, params.m_eventPackage, PSafeReadWrite));
   
   // If there is already a request with this URL and method, 
   // then update it with the new information
@@ -1112,7 +1149,7 @@ bool SIPEndPoint::Subscribe(const SIPSubscribe::Params & params, PString & aor)
 
   aor = handler->GetAddressOfRecord().AsString();
 
-  return handler->SendRequest(SIPHandler::Subscribing);
+  return handler->ActivateState(SIPHandler::Subscribing);
 }
 
 
@@ -1125,13 +1162,12 @@ bool SIPEndPoint::Unsubscribe(SIPSubscribe::PredefinedPackages eventPackage, con
 bool SIPEndPoint::Unsubscribe(const PString & eventPackage, const PString & to)
 {
   // Create the SIPHandler structure
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_SUBSCRIBE, eventPackage, PSafeReadOnly);
-  if (handler == NULL) {
-    PTRACE(1, "SIP\tCould not find active SUBSCRIBE for " << to);
-    return PFalse;
-  }
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_SUBSCRIBE, eventPackage, PSafeReference);
+  if (handler != NULL)
+    return handler->ActivateState(SIPHandler::Unsubscribing);
 
-  return handler->SendRequest(SIPHandler::Unsubscribing);
+  PTRACE(1, "SIP\tCould not find active SUBSCRIBE of " << eventPackage << " package to " << to);
+  return PFalse;
 }
 
 
@@ -1143,16 +1179,16 @@ bool SIPEndPoint::UnsubcribeAll(SIPSubscribe::PredefinedPackages eventPackage)
 
 bool SIPEndPoint::UnsubcribeAll(const PString & eventPackage)
 {
-  bool ok = true;
+  bool atLeastOne = false;
 
-  for (PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
-    if (handler->GetMethod() == SIP_PDU::Method_SUBSCRIBE && handler->GetEventPackage() == eventPackage) {
-      if (!handler->SendRequest(SIPHandler::Unsubscribing))
-        ok = false;
-    }
+  for (PSafePtr<SIPHandler> handler = activeSIPHandlers.GetFirstHandler(); handler != NULL; ++handler) {
+    if (handler->GetMethod() == SIP_PDU::Method_SUBSCRIBE &&
+        handler->GetEventPackage() == eventPackage &&
+        handler->ActivateState(SIPHandler::Unsubscribing))
+      atLeastOne = true;
   }
 
-  return ok;
+  return atLeastOne;
 }
 
 
@@ -1166,7 +1202,7 @@ bool SIPEndPoint::Notify(const SIPURL & aor, const PString & eventPackage, const
 {
   bool atLeastOne = false;
 
-  for (PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReadWrite); handler != NULL; ++handler) {
+  for (PSafePtr<SIPHandler> handler = activeSIPHandlers.GetFirstHandler(); handler != NULL; ++handler) {
     if (handler->GetMethod() == SIP_PDU::Method_NOTIFY &&
         handler->GetAddressOfRecord() == aor &&
         handler->GetEventPackage() == eventPackage &&
@@ -1180,7 +1216,7 @@ bool SIPEndPoint::Notify(const SIPURL & aor, const PString & eventPackage, const
 PBoolean SIPEndPoint::Message(const PString & to, const PString & body)
 {
   // Create the SIPHandler structure
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_MESSAGE, PSafeReadOnly);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_MESSAGE, PSafeReadWrite);
 
   // Otherwise create a new request with this method type
   if (handler != NULL)
@@ -1190,13 +1226,13 @@ PBoolean SIPEndPoint::Message(const PString & to, const PString & body)
     activeSIPHandlers.Append(handler);
   }
 
-  return handler->SendRequest(SIPHandler::Subscribing);
+  return handler->ActivateState(SIPHandler::Subscribing);
 }
 
 PBoolean SIPEndPoint::Message(const PString & to, const PString & body, const PString & remoteContact, const PString & callId)
 {
   // Create the SIPHandler structure
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(callId, PSafeReadOnly);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(callId, PSafeReadWrite);
   
   // Otherwise create a new request with this method type
   if (handler != NULL)
@@ -1206,7 +1242,7 @@ PBoolean SIPEndPoint::Message(const PString & to, const PString & body, const PS
     activeSIPHandlers.Append(handler);
   }
 
-  return handler->SendRequest(SIPHandler::Subscribing);
+  return handler->ActivateState(SIPHandler::Subscribing);
 }
 
 
@@ -1227,11 +1263,11 @@ void SIPEndPoint::OnMessageReceived (const SIPURL & /*from*/, const PString & /*
 
 PBoolean SIPEndPoint::Ping(const PString & to)
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_PING, PSafeReadOnly);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_PING, PSafeReference);
   if (handler == NULL)
     handler = new SIPPingHandler(*this, to);
 
-  return handler->SendRequest(SIPHandler::Subscribing);
+  return handler->ActivateState(SIPHandler::Subscribing);
 }
 
 
@@ -1247,7 +1283,7 @@ bool SIPEndPoint::Publish(const SIPSubscribe::Params & params, const PString & b
 
   aor = handler->GetAddressOfRecord().AsString();
 
-  return handler->SendRequest(params.m_expire != 0 ? SIPHandler::Subscribing : SIPHandler::Unsubscribing);
+  return handler->ActivateState(params.m_expire != 0 ? SIPHandler::Subscribing : SIPHandler::Unsubscribing);
 }
 
 
@@ -1361,7 +1397,7 @@ unsigned SIPEndPoint::GetAllowedMethods() const
 
 PBoolean SIPEndPoint::GetAuthentication(const PString & authRealm, PString & realm, PString & user, PString & password) 
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByAuthRealm(authRealm, PString::Empty(), PSafeReadOnly);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByAuthRealm(authRealm, user, PSafeReadOnly);
   if (handler == NULL)
     return PFalse;
 
@@ -1374,9 +1410,14 @@ PBoolean SIPEndPoint::GetAuthentication(const PString & authRealm, PString & rea
 
 SIPURL SIPEndPoint::GetRegisteredPartyName(const SIPURL & url, const OpalTransport & transport)
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByDomain(url.GetHostName(), SIP_PDU::Method_REGISTER, PSafeReadOnly);
-  if (handler == NULL) 
-    return GetDefaultRegisteredPartyName(transport);
+  // Look up by the full URL first in case of multiple registrations to the same domain.
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(url.AsString(), SIP_PDU::Method_REGISTER, PSafeReadOnly);
+  if (handler == NULL) {
+    // Precise AOR not found, locate the name used for the domain.
+    handler = activeSIPHandlers.FindSIPHandlerByDomain(url.GetHostName(), SIP_PDU::Method_REGISTER, PSafeReadOnly);
+    if (handler == NULL) 
+      return GetDefaultRegisteredPartyName(transport);
+  }
 
   return handler->GetAddressOfRecord();
 }
@@ -1426,16 +1467,18 @@ SIPURL SIPEndPoint::GetDefaultRegisteredPartyName(const OpalTransport & transpor
 }
 
 
-SIPURL SIPEndPoint::GetContactURL(const OpalTransport &transport, const PString & userName, const PString & host)
+SIPURL SIPEndPoint::GetContactURL(const OpalTransport &transport, const SIPURL & localURI)
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByDomain(host, SIP_PDU::Method_REGISTER, PSafeReadOnly);
-  if (handler != NULL) {
-    OpalTransport * handlerTransport = handler->GetTransport();
-    if (handlerTransport != NULL)
-      return GetLocalURL(*handlerTransport, handler->GetAddressOfRecord().GetUserName());
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(localURI.AsString(), SIP_PDU::Method_REGISTER, PSafeReadOnly);
+  if (handler == NULL) {
+    // Precise AOR not found, locate the name used for the domain.
+    handler = activeSIPHandlers.FindSIPHandlerByDomain(localURI.GetHostName(), SIP_PDU::Method_REGISTER, PSafeReadOnly);
+    if (handler == NULL)
+      return GetLocalURL(transport, localURI.GetUserName());
   }
 
-  return GetLocalURL(transport, userName);
+  OpalTransport * handlerTransport = handler->GetTransport();
+  return GetLocalURL(handlerTransport != NULL ? *handlerTransport : transport, handler->GetAddressOfRecord().GetUserName());
 }
 
 
@@ -1481,57 +1524,83 @@ void SIPEndPoint::OnRTPStatistics(const SIPConnection & connection,
   manager.OnRTPStatistics(connection, session);
 }
 
-SIPEndPoint::SIP_PDU_Thread::SIP_PDU_Thread(PThreadPoolBase & _pool)
-  : PThreadPoolWorkerBase(_pool)
-{
-  SetPriority(HighestPriority);
+///////////////////////////////////////////////////////////////////////////////////////////////////
+
+SIPEndPoint::PDUThreadPool::WorkerThreadBase * SIPEndPoint::PDUThreadPool::CreateWorkerThread()
+{ 
+  return new SIP_PDU_Thread(*this); 
 }
 
-unsigned SIPEndPoint::SIP_PDU_Thread::GetWorkSize() const { return pduQueue.size(); }
 
-void SIPEndPoint::SIP_PDU_Thread::OnAddWork(SIP_PDU_Work * work)
+SIPEndPoint::SIP_PDU_Thread::SIP_PDU_Thread(PDUThreadPool & pool_)
+  : PDUThreadPool::WorkerThread(pool_)
 {
-  PWaitAndSignal m(mutex);
-  pduQueue.push(work);
-  if (pduQueue.size() == 1)
-    sync.Signal();
+  SetPriority(HighPriority);
+}
+
+
+unsigned SIPEndPoint::SIP_PDU_Thread::GetWorkSize() const 
+{ 
+  return m_pduQueue.size(); 
+}
+
+
+void SIPEndPoint::SIP_PDU_Thread::AddWork(SIP_PDU_Work * work)
+{
+  PWaitAndSignal m(m_workerMutex);
+  m_pduQueue.push(work);
+  if (m_pduQueue.size() == 1)
+    m_sync.Signal();
+}
+
+
+void SIPEndPoint::SIP_PDU_Thread::RemoveWork(SIP_PDU_Work * work)
+{
+  m_workerMutex.Wait();
+  m_pduQueue.pop();
+  m_workerMutex.Signal();
+
+  delete work;
 }
 
-void SIPEndPoint::SIP_PDU_Thread::OnRemoveWork(SIP_PDU_Work *) 
-{ }
 
 void SIPEndPoint::SIP_PDU_Thread::Shutdown()
 {
-  shutdown = true;
-  sync.Signal();
+  m_shutdown = true;
+  m_sync.Signal();
 }
 
 
 void SIPEndPoint::SIP_PDU_Thread::Main()
 {
-  while (!shutdown) {
-    mutex.Wait();
-    if (pduQueue.size() == 0) {
-      mutex.Signal();
-      sync.Wait();
+  while (!m_shutdown) {
+
+    // get the work
+    m_workerMutex.Wait();
+    SIP_PDU_Work * work = m_pduQueue.size() > 0 ? m_pduQueue.front() : NULL;
+    m_workerMutex.Signal();
+
+    // wait for work to become available
+    if (work == NULL) {
+      m_sync.Wait();
       continue;
     }
 
-    SIP_PDU_Work * work = pduQueue.front();
-    pduQueue.pop();
-    mutex.Signal();
-
+    // process the work
     work->OnReceivedPDU();
-    delete work;
+
+    // indicate work is now free
+    m_pool.RemoveWork(work);
   }
 }
 
-
 SIPEndPoint::SIP_PDU_Work::SIP_PDU_Work(SIPEndPoint & ep, const PString & token, SIP_PDU * pdu)
   : m_endpoint(ep)
   , m_token(token)
   , m_pdu(pdu)
 {
+  PTRACE(4, "SIP\tQueueing PDU \"" << *m_pdu << "\", transaction="
+         << m_pdu->GetTransactionID() << ", token=" << m_token);
 }
 
 
@@ -1547,20 +1616,27 @@ void SIPEndPoint::SIP_PDU_Work::OnReceivedPDU()
     return;
 
   if (m_pdu->GetMethod() == SIP_PDU::NumMethods) {
-    PSafePtr<SIPTransaction> transaction = m_endpoint.GetTransaction(m_pdu->GetTransactionID(), PSafeReference);
+    PString transactionID = m_pdu->GetTransactionID();
+    PTRACE(4, "SIP\tHandling PDU \"" << *m_pdu << "\" for transaction=" << transactionID);
+    PSafePtr<SIPTransaction> transaction = m_endpoint.GetTransaction(transactionID, PSafeReference);
     if (transaction != NULL)
       transaction->OnReceivedResponse(*m_pdu);
     else {
-      PTRACE(3, "SIP\tCannot find transaction for response");
+      PTRACE(2, "SIP\tCannot find transaction " << transactionID << " for response PDU \"" << *m_pdu << '"');
     }
-    return;
   }
 
-  if (PAssert(!m_token.IsEmpty(), PInvalidParameter)) {
+  else if (PAssert(!m_token.IsEmpty(), PInvalidParameter)) {
+    PTRACE(4, "SIP\tHandling PDU \"" << *m_pdu << "\" for token=" << m_token);
     PSafePtr<SIPConnection> connection = m_endpoint.GetSIPConnectionWithLock(m_token, PSafeReference);
     if (connection != NULL) 
       connection->OnReceivedPDU(*m_pdu);
+    else {
+      PTRACE(2, "SIP\tCannot find connection for PDU \"" << *m_pdu << "\" using token=" << m_token);
+    }
   }
+
+  PTRACE(4, "SIP\tHandled PDU \"" << *m_pdu << '"');
 }
 
 
@@ -1574,9 +1650,9 @@ SIPEndPoint::InterfaceMonitor::InterfaceMonitor(SIPEndPoint & ep, PINDEX priorit
 void SIPEndPoint::InterfaceMonitor::OnAddInterface(const PIPSocket::InterfaceEntry &)
 {
   if (priority == SIPEndPoint::LowPriority) {
-    for (PSafePtr<SIPHandler> handler(m_endpoint.activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
+    for (PSafePtr<SIPHandler> handler = m_endpoint.activeSIPHandlers.GetFirstHandler(); handler != NULL; ++handler) {
       if (handler->GetState() == SIPHandler::Unavailable)
-        handler->SendRequest(SIPHandler::Restoring);
+        handler->ActivateState(SIPHandler::Restoring);
     }
   } else {
     // special case if interface filtering is used: A new interface may 'hide' the old interface.
@@ -1587,7 +1663,7 @@ void SIPEndPoint::InterfaceMonitor::OnAddInterface(const PIPSocket::InterfaceEnt
     // the transport will still listen on the old interface only. Therefore, clear the
     // socket binding BEFORE the monitored sockets update their interfaces.
     if (PInterfaceMonitor::GetInstance().HasInterfaceFilter()) {
-      for (PSafePtr<SIPHandler> handler(m_endpoint.activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
+      for (PSafePtr<SIPHandler> handler = m_endpoint.activeSIPHandlers.GetFirstHandler(PSafeReadOnly); handler != NULL; ++handler) {
         OpalTransport *transport = handler->GetTransport();
         if (transport != NULL) {
           PString iface = transport->GetInterface();
@@ -1614,12 +1690,12 @@ void SIPEndPoint::InterfaceMonitor::OnAddInterface(const PIPSocket::InterfaceEnt
 void SIPEndPoint::InterfaceMonitor::OnRemoveInterface(const PIPSocket::InterfaceEntry & entry)
 {
   if (priority == SIPEndPoint::LowPriority) {
-    for (PSafePtr<SIPHandler> handler(m_endpoint.activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
+    for (PSafePtr<SIPHandler> handler = m_endpoint.activeSIPHandlers.GetFirstHandler(PSafeReadOnly); handler != NULL; ++handler) {
       if (handler->GetState() == SIPHandler::Subscribed &&
           handler->GetTransport() != NULL &&
           handler->GetTransport()->GetInterface().Find(entry.GetName()) != P_MAX_INDEX) {
         handler->GetTransport()->SetInterface(PString::Empty());
-        handler->SendRequest(SIPHandler::Refreshing);
+        handler->ActivateState(SIPHandler::Refreshing);
       }
     }
   }
