@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22097 $
+ * $Revision: 23075 $
  * $Author: rjongbloed $
- * $Date: 2009-02-21 04:55:10 +0000 (Sat, 21 Feb 2009) $
+ * $Date: 2009-07-05 05:39:52 +0000 (Sun, 05 Jul 2009) $
  */
 
 #include <ptlib.h>
@@ -479,12 +479,18 @@ PBoolean SIPURL::AdjustToDNS(PINDEX entry)
   // Do the SRV lookup, if fails, then we actually return TRUE so outer loops
   // can use the original host name value.
   PIPSocketAddressAndPortVector addrs;
-  if (!PDNS::LookupSRV(GetHostName(), "_sip._" + paramVars("transport", "udp"), GetPort(), addrs))
+  if (!PDNS::LookupSRV(GetHostName(), "_sip._" + paramVars("transport", "udp"), GetPort(), addrs)) {
+    PTRACE(4, "SIP\tNo SRV record found.");
     return PTrue;
+  }
 
   // Got the SRV list, return FALSE if outer loop has got to the end of it
-  if (entry >= (PINDEX)addrs.size())
+  if (entry >= (PINDEX)addrs.size()) {
+    PTRACE(4, "SIP\tRan out of SRV records at entry " << entry);
     return PFalse;
+  }
+
+  PTRACE(4, "SIP\tAttempting SRV record entry " << entry << ": " << addrs[entry].AsString());
 
   // Adjust our host and port to what the DNS SRV record says
   SetHostName(addrs[entry].GetAddress().AsString());
@@ -578,7 +584,7 @@ PBoolean SIPMIMEInfo::IsContentLengthPresent() const
 }
 
 
-PString SIPMIMEInfo::GetContentType() const
+PCaselessString SIPMIMEInfo::GetContentType() const
 {
   return GetString("Content-Type");
 }
@@ -590,7 +596,7 @@ void SIPMIMEInfo::SetContentType(const PString & v)
 }
 
 
-PString SIPMIMEInfo::GetContentEncoding() const
+PCaselessString SIPMIMEInfo::GetContentEncoding() const
 {
   return GetString("Content-Encoding");
 }
@@ -1045,8 +1051,10 @@ void SIPMIMEInfo::GetProductInfo(OpalProductInfo & info)
   PCaselessString str = GetUserAgent();
   if (str.IsEmpty()) {
     str = GetString("Server");
-    if (str.IsEmpty())
+    if (str.IsEmpty()) {
+      PTRACE(4, "SIP\tNo User-Agent or Server fields, Product Info unknown.");
       return; // Have nothing, change nothing
+    }
   }
 
   // This is not strictly correct according to he BNF, but we cheat
@@ -1060,6 +1068,7 @@ void SIPMIMEInfo::GetProductInfo(OpalProductInfo & info)
     info.name = str;
     info.vendor = info.version = PString::Empty();
     info.manufacturerCode = info.t35Extension = info.t35CountryCode = 0;
+    PTRACE(4, "SIP\tProduct Info: name=\"" << str << '"');
     return;
   }
 
@@ -1071,6 +1080,10 @@ void SIPMIMEInfo::GetProductInfo(OpalProductInfo & info)
   info.version = str(endFirstToken+1, endSecondToken);
   info.vendor = GetOrganization();
   info.comments = str.Mid(endSecondToken+1).Trim();
+  PTRACE(4, "SIP\tProduct Info: name=\"" << info.name << "\","
+                           " version=\"" << info.version << "\","
+                            " vendor=\"" << info.vendor << "\","
+                          " comments=\"" << info.comments << '"');
 }
 
 
@@ -1195,7 +1208,10 @@ void SIPMIMEInfo::GetAlertInfo(PString & info, int & appearance)
 void SIPMIMEInfo::SetAlertInfo(const PString & info, int appearance)
 {
   if (appearance < 0) {
-    SetAt("Alert-Info", info);
+    if (info.IsEmpty())
+      RemoveAt("Alert-Info");
+    else
+      SetAt("Alert-Info", info);
     return;
   }
 
@@ -1215,7 +1231,13 @@ static bool LocateFieldParameter(const PString & fieldValue, const PString & par
   PINDEX semicolon = (PINDEX)-1;
   while ((semicolon = fieldValue.Find(';', semicolon+1)) != P_MAX_INDEX) {
     start = fieldValue.FindSpan("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-.!%*_+`'~", semicolon+1);
-    if (start != P_MAX_INDEX && fieldValue[start] == '=' && (fieldValue(semicolon+1, start-1) *= paramName)) {
+    if (start > fieldValue.GetLength()) {
+      if (fieldValue(semicolon+1, start) *= paramName) {
+        end = start;
+        return true;
+      }
+    }
+    else if ((fieldValue[start] == '=') && (fieldValue(semicolon+1, start-1) *= paramName)) {
       start++;
       end = fieldValue.FindOneOf("()<>@,;:\\\"/[]?{}= \t", start)-1;
       return true;
@@ -1241,8 +1263,11 @@ PString SIPMIMEInfo::InsertFieldParameter(const PString & fieldValue,
 {
 
   PINDEX start, end;
-  if (LocateFieldParameter(fieldValue, paramName, start, end))
-    return fieldValue.Left(start) + newValue + fieldValue.Mid(end+1);
+  if (LocateFieldParameter(fieldValue, paramName, start, end)) {
+    if (start != end)
+      return fieldValue.Left(start) + newValue + fieldValue.Mid(end+1);
+    return fieldValue.Left(start) + "=" + newValue + fieldValue.Mid(end+1);
+  }
 
   PStringStream newField;
   newField << fieldValue << ';' << paramName << '=' << newValue;
@@ -1799,7 +1824,7 @@ void SIP_PDU::Construct(Methods meth,
   PINDEX pos = localPartyName.Find('@');
   if (pos != P_MAX_INDEX)
     localPartyName = localPartyName.Left(pos);
-  SIPURL contact = endpoint.GetContactURL(transport, localPartyName, SIPURL(connection.GetRemotePartyAddress()).GetHostName());
+  SIPURL contact = endpoint.GetContactURL(transport, SIPURL(localPartyName, connection.GetRemotePartyAddress(), endpoint.GetDefaultSignalPort()));
   contact.Sanitise(meth != Method_INVITE ? SIPURL::ContactURI : SIPURL::RouteURI);
   mime.SetContact(contact);
 
@@ -1955,14 +1980,29 @@ bool SIP_PDU::SendResponse(OpalTransport & transport, SIP_PDU & response, SIPEnd
       viaAddress = param;
 
     // received is present
+    bool received = false;
     param = SIPMIMEInfo::ExtractFieldParameter(viaList.front(), "received");
-    if (!param.IsEmpty()) 
+    if (!param.IsEmpty()) {
       viaAddress = param;
+      received = true;
+    }
 
-    // rport is present
-    param = SIPMIMEInfo::ExtractFieldParameter(viaList.front(), "rport");
-    if (!param.IsEmpty()) 
-      viaPort = param;
+    // rport is present. be careful to distinguish between not present and empty
+    PIPSocket::Address remoteIp;
+    WORD remotePort;
+    transport.GetLastReceivedAddress().GetIpAndPort(remoteIp, remotePort);
+    {
+      PINDEX start, end;
+      if (LocateFieldParameter(viaList.front(), "rport", start, end)) {
+        param = viaList.front()(start, end);
+        if (!param.IsEmpty()) 
+          viaPort = param;
+        else
+          viaPort = remotePort;
+        if (!received) 
+          viaAddress = remoteIp.AsString();
+      }
+    }
 
     newAddress = OpalTransportAddress(viaAddress+":"+viaPort, defaultPort, (proto *= "TCP") ? "tcp$" : "udp$");
   }
@@ -2173,6 +2213,7 @@ PBoolean SIP_PDU::Write(OpalTransport & transport, const OpalTransportAddress &
       PTRACE(1, "SIP\tCannot use remote address " << remoteAddress << " for transport " << transport);
       return false;
     }
+    PTRACE(4, "SIP\tSet new remote address " << remoteAddress << " for transport " << transport);
   }
 
   PString oldInterface = transport.GetInterface();
@@ -2181,6 +2222,7 @@ PBoolean SIP_PDU::Write(OpalTransport & transport, const OpalTransportAddress &
       PTRACE(1, "SIP\tCannot use local interface \"" << localInterface << "\" for transport " << transport);
       return false;
     }
+    PTRACE(4, "SIP\tSet new interface " << localInterface << " for transport " << transport);
   }
 
   mime.SetCompactForm(false);
@@ -2276,8 +2318,8 @@ PString SIP_PDU::GetTransactionID() const
 
 SDPSessionDescription * SIP_PDU::GetSDP()
 {
-  if (m_SDP == NULL && (mime.GetContentType() *= "application/sdp")) {
-    m_SDP = new SDPSessionDescription();
+  if (m_SDP == NULL && mime.GetContentType() == "application/sdp") {
+    m_SDP = new SDPSessionDescription(0, 0, OpalTransportAddress());
     if (!m_SDP->Decode(entityBody)) {
       delete m_SDP;
       m_SDP = NULL;
@@ -2288,6 +2330,13 @@ SDPSessionDescription * SIP_PDU::GetSDP()
 }
 
 
+void SIP_PDU::SetSDP(SDPSessionDescription * sdp)
+{
+  delete m_SDP;
+  m_SDP = sdp;
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////////
 
 SIPDialogContext::SIPDialogContext()
@@ -2429,7 +2478,7 @@ SIPTransaction::SIPTransaction(SIPEndPoint & ep,
   , transport(trans)
 {
   Construct(minRetryTime, maxRetryTime);
-  PTRACE(4, "SIP\tTransaction " << mime.GetCSeq() << " created.");
+  PTRACE(4, "SIP\tTransaction created.");
 }
 
 
@@ -2442,7 +2491,7 @@ SIPTransaction::SIPTransaction(SIPConnection & conn,
 {
   connection = &conn;
   Construct();
-  PTRACE(4, "SIP\tTransaction " << mime.GetCSeq() << " created.");
+  PTRACE(4, "SIP\t" << meth << " transaction id=" << GetTransactionID() << " created.");
 }
 
 
@@ -2474,8 +2523,9 @@ void SIPTransaction::Construct(Methods method, SIPDialogContext & dialog)
 
 SIPTransaction::~SIPTransaction()
 {
-  PTRACE_IF(1, state < Terminated_Success, "SIP\tDestroying transaction " << mime.GetCSeq() << " which is not yet terminated.");
-  PTRACE(4, "SIP\tTransaction " << mime.GetCSeq() << " destroyed.");
+  PTRACE_IF(1, state < Terminated_Success, "SIP\tDestroying transaction id="
+            << GetTransactionID() << " which is not yet terminated.");
+  PTRACE(4, "SIP\tTransaction id=" << GetTransactionID() << " destroyed.");
 }
 
 
@@ -2556,11 +2606,14 @@ PBoolean SIPTransaction::Cancel()
   PSafeLockReadWrite lock(*this);
 
   if (state == NotStarted || state >= Cancelling) {
-    PTRACE(3, "SIP\tTransaction " << mime.GetCSeq() << " cannot be cancelled as in state " << state);
+    PTRACE(3, "SIP\t" << GetMethod() << " transaction id=" << GetTransactionID() << " cannot be cancelled as in state " << state);
     return PFalse;
   }
 
-  PTRACE(4, "SIP\tTransaction " << mime.GetCSeq() << " cancelled.");
+  PTRACE(4, "SIP\t" << GetMethod() << " transaction id=" << GetTransactionID() << " cancelled.");
+  state = Cancelling;
+  retry = 0;
+  retryTimer = retryTimeoutMin;
   completionTimer = endpoint.GetPduCleanUpTimeout();
   return ResendCANCEL();
 }
@@ -2569,7 +2622,9 @@ PBoolean SIPTransaction::Cancel()
 void SIPTransaction::Abort()
 {
   if (LockReadWrite()) {
-    SetTerminated(Terminated_Aborted);
+    PTRACE(4, "SIP\t" << GetMethod() << " transaction id=" << GetTransactionID() << " aborted.");
+    if (!IsCompleted())
+      SetTerminated(Terminated_Aborted);
     UnlockReadWrite();
   }
 }
@@ -2598,16 +2653,7 @@ bool SIPTransaction::ResendCANCEL()
   PStringList viaList = mime.GetViaList();
   cancel.GetMIME().SetVia(viaList.front());
 
-  if (!SendPDU(cancel))
-    return false;
-
-  if (state < Cancelling) {
-    state = Cancelling;
-    retry = 0;
-    retryTimer = retryTimeoutMin;
-  }
-
-  return true;
+  return SendPDU(cancel);
 }
 
 
@@ -2645,7 +2691,7 @@ PBoolean SIPTransaction::OnReceivedResponse(SIP_PDU & response)
      */
   if (IsInProgress()) {
     if (response.GetStatusCode()/100 == 1) {
-      PTRACE(3, "SIP\tTransaction " << cseq << " proceeding.");
+      PTRACE(3, "SIP\t" << GetMethod() << " transaction id=" << GetTransactionID() << " proceeding.");
 
       if (state == Trying)
         state = Proceeding;
@@ -2662,7 +2708,7 @@ PBoolean SIPTransaction::OnReceivedResponse(SIP_PDU & response)
         completionTimer = endpoint.GetNonInviteTimeout();
     }
     else {
-      PTRACE(3, "SIP\tTransaction " << cseq << " completed.");
+      PTRACE(3, "SIP\t" << GetMethod() << " transaction id=" << GetTransactionID() << " completed.");
       state = Completed;
       statusCode = response.GetStatusCode();
     }
@@ -2695,32 +2741,32 @@ void SIPTransaction::OnRetry(PTimer &, INT)
 {
   PSafeLockReadWrite lock(*this);
 
-  if (!lock.IsLocked() || (state != Trying && state != Cancelling))
+  if (!lock.IsLocked() || state > Cancelling || (state == Proceeding && method == Method_INVITE))
     return;
 
   retry++;
 
-  PTRACE(3, "SIP\tTransaction " << mime.GetCSeq() << " timeout, making retry " << retry);
-
   if (retry >= endpoint.GetMaxRetries()) {
     SetTerminated(Terminated_RetriesExceeded);
     return;
   }
 
-  if (state == Cancelling) {
-    if (!ResendCANCEL())
-      return;
-  }
+  if (state > Trying)
+    retryTimer = retryTimeoutMax;
   else {
-    if (!SendPDU(*this))
-      return;
+    PTimeInterval timeout = retryTimeoutMin*(1<<retry);
+    if (timeout > retryTimeoutMax)
+      timeout = retryTimeoutMax;
+    retryTimer = timeout;
   }
 
-  PTimeInterval timeout = retryTimeoutMin*(1<<retry);
-  if (timeout > retryTimeoutMax)
-    retryTimer = retryTimeoutMax;
+  PTRACE(3, "SIP\t" << GetMethod() << " transaction id=" << GetTransactionID()
+         << " timeout, making retry " << retry << ", timeout " << retryTimer);
+
+  if (state == Cancelling)
+    ResendCANCEL();
   else
-    retryTimer = timeout;
+    SendPDU(*this);
 }
 
 
@@ -2784,7 +2830,7 @@ void SIPTransaction::SetTerminated(States newState)
 
   if (state >= Terminated_Success) {
     PTRACE_IF(3, newState != Terminated_Success, "SIP\tTried to set state " << StateNames[newState] 
-              << " for transaction " << mime.GetCSeq()
+              << " for " << GetMethod() << " transaction id=" << GetTransactionID()
               << " but already terminated ( " << StateNames[state] << ')');
     return;
   }
@@ -2792,7 +2838,8 @@ void SIPTransaction::SetTerminated(States newState)
   States oldState = state;
   
   state = newState;
-  PTRACE(3, "SIP\tSet state " << StateNames[newState] << " for transaction " << mime.GetCSeq());
+  PTRACE(3, "SIP\tSet state " << StateNames[newState] << " for "
+         << GetMethod() << " transaction id=" << GetTransactionID());
 
   // Transaction failed, tell the endpoint
   if (state > Terminated_Success) {
@@ -2841,12 +2888,6 @@ SIPInvite::SIPInvite(SIPConnection & connection, OpalTransport & transport, cons
   SetAllow(connection.GetEndPoint().GetAllowedMethods());
   mime.SetProductInfo(connection.GetEndPoint().GetUserAgent(), connection.GetProductInfo());
 
-  m_SDP = new SDPSessionDescription();
-  if (!connection.OnSendSDP(false, rtpSessions, *m_SDP)) {
-    delete m_SDP;
-    m_SDP = NULL;
-  }
-
   connection.OnCreatingINVITE(*this);
 }
 
@@ -2893,6 +2934,7 @@ SIPRegister::Params::Params()
   , m_restoreTime(30)
   , m_minRetryTime(PMaxTimeInterval)
   , m_maxRetryTime(PMaxTimeInterval)
+  , m_userData(NULL)
 {
 }
 
@@ -2989,6 +3031,7 @@ SIPSubscribe::Params::Params(PredefinedPackages pkg)
   , m_restoreTime(30)
   , m_minRetryTime(PMaxTimeInterval)
   , m_maxRetryTime(PMaxTimeInterval)
+  , m_userData(NULL)
 {
 }
 
